/*
*  Name:      abstract_application.hh
*  Author:    Angel Jimenez Jimenez
*             Rafael Jesus Alcantara Perez
*  Summary:   CGI base application
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_NET_CGI_ABSTRACT_APPLICATION__
#define _MPCL_NET_CGI_ABSTRACT_APPLICATION__

#include <iostream>
#include "../../abstract_application.hh"
#include "../../memory/smart_pointer.hh"
#include "../../text/html/base_page.hh"
#include "../../text/html/form.hh"
#include "../../text/string.hh"
#include "../../util/collection/map.hh"
#include "config_processor.hh"
#include "cookie.hh"
#include "session.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for NET.
  namespace net
  {

    ///  Name-space for CGI.
    namespace cgi
    {

      using text::html::QKTBasePage;
      using text::html::QTBasePage;
      using text::html::TForm;

      /**
      *  CGI application.
      *
      *  Modus operandi:
      *  1. It parses form data in every kind of input (GET, POST).
      *  2. If the CGI was called  from a form inside a  build-able page,  then the
      *     this  page is  built and its form is  filled with the  old values taken
      *     from input  form data.  To get the  source page or  form,  you must use
      *     \a getSourcePage() or \a getSourceForm().
      *  3. Nevertheless, the application must make a response, that can be another
      *     page or a redirection.  This must be done in \a processRequest(), using
      *     \a setResponse() or \a redirect().
      *
      *  It uses variable PATH_INFO to know from which page it has been called.
      *  Example:
      *
      *    /page-identifier/form-identifier
      *
      *  @see TBasePage, TConfigProcessor.
      *  @see #getSourcePage(), #getSourceForm(), #setResponse(), #redirect().
      */
      class TAbstractApplication : public mpcl::TAbstractApplication
      {

        private:

          ///  True if the CGI must be redirected to \a yTargetUrl.
          bool gIsRedirected;

          ///  Pointer to source form from page pointed by \a qtSourcePage.
          TForm*   ptSourceForm;

          ///  Smart pointer to source page.
          QTBasePage   qtSourcePage;

          ///  Smart pointer to the target page.
          QTBasePage   qtTargetPage;

          ///  Target URL for redirection.
          TString   yTargetUrl;


        protected:

          ///  Target OSTREAM instance.
          std::basic_ostream<char>*   ptTargetOstream;

          ///  Configuration handler.
          TConfigProcessor   tConfigProcessor;

          ///  Session instance.
          TSession<TCookie>   tSession;


        private:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds the page where the CGI was called (if any).
          void buildSourcePage (void);


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance.
          *  @param pkcNAME          Application name.
          *  @param pkcRELEASE       Application release.
          *  @param iPARAMETER_COUNT Parameter count.
          *  @param ppkcPARAMETERS   Parameter array.
          */
          TAbstractApplication ( const char*  pkcNAME          ,
                                 const char*  pkcRELEASE       ,
                                 int          iPARAMETER_COUNT ,
                                 const char** ppkcPARAMETERS   )  :
            mpcl::TAbstractApplication (pkcNAME, pkcRELEASE)      ,
            gIsRedirected      (false)                            ,
            ptSourceForm       (NULL)                             ,
            qtSourcePage       ()                                 ,
            qtTargetPage       ()                                 ,
            yTargetUrl         ()                                 ,
            ptTargetOstream    (&std::cout)                       ,
            tConfigProcessor   (iPARAMETER_COUNT, ppkcPARAMETERS) ,
            tSession           ()
          {
            TString                           yHttpCookieValue;
            std::basic_istringstream<char>*   ptIstrstream;

            //
            //  Loads cookies from HTTP environment variable.
            //
            yHttpCookieValue = tConfigProcessor.environmentValue ("HTTP_COOKIE");
            ptIstrstream     = new std::basic_istringstream<char> (yHttpCookieValue.c_str());
            *ptIstrstream >> tSession;
            delete ptIstrstream;
          }

          /**
          *  Builds the document named  \a rkyDOCUMENT_IDENTIFIER.  It returns the
          *  unique tag called \a rkyTAG_IDENTIFIER. If the tag doesn't exists, it
          *  creates it else returns the existent one.
          *  @param rkyPAGE_IDENTIFIER Page identifier.
          *  @return The built page.
          */
          virtual QTBasePage buildPage (const TString& rkyPAGE_IDENTIFIER) = 0;

          /**
          *  Initializes application. In a console application, this means that no
          *  output  can  be done (other than exceptions or errors). It  processes
          *  command-line options and  read  the configuration files (if any).  To
          *  start the application, this method must be called before.
          *  @see #buildSourcePage(), #start().
          */
          void initialize (void) = 0;

          /**
          *  Processes  the request and sets the response (using  setResponse()) or
          *  redirects to another URL (using redirect()).
          *  @see #setResponse(), #redirect().
          */
          virtual void processRequest (void) = 0;

          /**
          *  Redirects the output of the CGI to this URL.
          *  @param rkyTARGET_URL Target URL.
          */
          void redirect (const TString& rkyTARGET_URL)
          {
            gIsRedirected = true;
            yTargetUrl    = rkyTARGET_URL;
          }

          /**
          *  Sets output stream for writting HTML objects.
          *  @param rtTARGET_OSTREAM Target stream.
          */
          void setOutput (std::basic_ostream<char>& rtTARGET_OSTREAM)
          {
            ptTargetOstream = &rtTARGET_OSTREAM;
          }

          /**
          *  Sets response document page.
          *  @param rkqtTARGET_PAGE Target page.
          */
          void setResponse (const QTBasePage& rkqtTARGET_PAGE)
          {
            gIsRedirected = false;
            qtTargetPage  = rkqtTARGET_PAGE;
          }

          /**
          *  Parse input form data,  preprocess  using preprocess() member function
          *  (that returns a HTML page),  and then process the  page with process()
          *  member function.
          *  @return Application exit status.
          */
          int start (void)
          {
            buildSourcePage();
            processRequest();
            writeResponse();
            return 0;
          }


        private:

          //
          //  S E L E C T O R S
          //

          /**
          *  Writes response document or redirects to another URL, depending on the
          *  use of setResponse() and redirect() methods.
          *  @see #setResponse(), #redirect().
          */
          void writeResponse (void) const;


        protected:

          /**
          *  Checks if the CGI has been called from another instance of this CGI.
          *  @return True if there is a source page.
          */
          bool hasSourcePage (void) const
          {
            return !qtSourcePage.isNull();
          }


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Gets the source page from where another  instance of this CGI has been
          *  called.
          *  @return The source page.
          */
          QKTBasePage getSourcePage (void) const
          {
            return QKTBasePage (qtSourcePage);
          }

          /**
          *  Gets the source form from where another  instance of this CGI has been
          *  called.
          *  @return The source form.
          */
          const TForm& getSourceForm (void) const
          {
            if ( !ptSourceForm )
            {
              throw TConstraintException ("form is null", __FILE__, __LINE__);
            }
            return *ptSourceForm;
          }

          /**
          *  Gets the response page, this is the page that the CGI will show (if it
          *  is not redirected).
          *  @return The response page.
          *  @see #setResponse(), #redirect().
          */
          QTBasePage getResponsePage (void) const
          {
            return qtTargetPage;
          }

          /**
          *  Gets the CGI path  and the CGI name  (see the CGI environment variable
          *  SCRIPT_NAME).
          *  @return CGI path and CGI name.
          */
          TString fullName (void) const
          {
            return tConfigProcessor.environmentValue ("SCRIPT_NAME");
          }

          TString programFileName (void) const
          {
            return tConfigProcessor.programFileName();
          }

      };  // class TAbstractApplication

    }  // namespace cgi

  }  // namespace net

}  // namespace mpcl


#endif  // not _MPCL_NET_CGI_ABSTRACT_APPLICATION__
