/*
*  Name:      condition.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base condition
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_CONDITION__
#define _MPCL_SYSTEM_CONDITION__

#include "mutex.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    /**
    *  Condition base class.
    *
    *  A condition (short for "condition variable") is a synchronization device
    *  that allows threads to suspend execution and relinquish the processors
    *  until some predicate on shared data is satisfied. The basic operations
    *  on conditions are: signal the condition (when the predicate becomes true),
    *  and wait for the condition, suspending the thread execution until another
    *  thread signals the condition.
    *
    *  A condition variable must always be associated with a mutex, to avoid the
    *  race condition where a thread prepares to wait on a condition variable and
    *  another thread signals the condition just before the first thread actually
    *  waits on it.
    *
    *  @see TBaseOperatingSystem, TBaseMutex.
    */
    class ICondition
    {

      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Destroys the instance.
        virtual ~ICondition (void) {}

        /**
        *  Atomically unlocks the associated mutex (as per member function
        *  TBaseMutex::unlock()) and waits for this condition  variable to
        *  be  signaled.  The thread execution is suspended and  does  not
        *  consume   any  CPU  time  until   this  condition  variable  is
        *  signaled.  That mutex must be locked by the calling  thread  on
        *  entrance  to  this  member function.  Before  returning  to the
        *  calling thread, this member function re-acquires mutex (as  per
        *  member function TBaseMutex::lock()). This member function seems
        *  like is prefixed with a TBaseMutex::unlock() and postfixed with
        *  a TBaseMutex::lock() over the related mutex.
        *  @see #signal(), #broadcast(), TBaseMutex.
        */
        virtual void wait (void) = 0;

        /**
        *  Atomically unlocks the related mutex and waits on this variable,
        *  as wait() does, but it also bounds  the  duration  of  the wait.
        *  If this condition  has  not  been  signaled  within  the  amount
        *  of time specified in parameters, the former mutex is re-acquired
        *  and  returns true.  In  any case the thread will re-acquire  the
        *  externally supplied mutex.
        *  @param zSECONDS     Seconds to wait for.
        *  @param zNANOSECONDS Nanoseconds to wait for.
        *  @return True if time has been reached or exceeded.
        *  @see #wait(), TBaseMutex.
        */
        virtual bool timedWait (size_t zSECONDS, size_t zNANOSECONDS = 0) = 0;

        /**
        *  Restarts one of the threads that are waiting on this condition variable
        *  If no threads are waiting on this, nothing happens.  If several threads
        *  are waiting on this, exactly one is restarted,  but it is not specified
        *  which.
        */
        virtual void signal (void) = 0;

        /**
        *  Restarts all the threads that are waiting on this condition variable.
        *  Nothing happens if no threads are waiting on this.
        */
        virtual void broadcast (void) = 0;

    };  // class ICondition

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_CONDITION__
