/*
*  Name:      system.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base operating system
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_SYSTEM__
#define _MPCL_SYSTEM_SYSTEM__

#include <cstdlib>
#include <vector>
#include "../text/string.hh"
#include "defs.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    using std::string;
    using std::vector;
    using text::TString;

    ///  Base class for an operating system wrapper.
    class ISystem
    {
    
      public:

        /**
        *  This  determines  the  importance of the message.  The  levels are, in
        *  order of decreasing importance:
        *  @see #writeLogEntry().
        */
        enum ELogLevel
        {
          eEmerg   , ///<  The system is unusable.
          eAlert   , ///<  The action must be taken immediately.
          eCrit    , ///<  Critical conditions.
          eErr     , ///<  Error conditions.
          eWarning , ///<  Warning conditions.
          eNotice  , ///<  A normal, but significant, condition.
          eInfo    , ///<  An informational message.
          eDebug     ///<  A debug-level message.
        };

        /**
        *  The facility is used  to  specify  what type of program is logging the
        *  message.  This lets the configuration file  specify that messages from
        *  different facilities will be handled differently.
        *  @see #writeLogEntry().
        */
        enum ELogFacility
        {
          eAuthPriv ,  ///<  Security/authorization messages (private).
          eCron     ,  ///<  Clock daemon (CRON and AT).
          eDaemon   ,  ///<  Other system daemons.
          eKern     ,  ///<  Kernel messages.
          eLocal0   ,  ///<  Reserved for local use.
          eLocal1   ,  ///<  Reserved for local use.
          eLocal2   ,  ///<  Reserved for local use.
          eLocal3   ,  ///<  Reserved for local use.
          eLocal4   ,  ///<  Reserved for local use.
          eLocal5   ,  ///<  Reserved for local use.
          eLocal6   ,  ///<  Reserved for local use.
          eLocal7   ,  ///<  Reserved for local use.
          eLpr      ,  ///<  Line printer subsystem.
          eMail     ,  ///<  Mail subsystem.
          eNews     ,  ///<  USENET news subsystem.
          eUser     ,  ///<  Generic user-level messages.
          eUucp        ///<  UUCP subsystem.
        };
        
        ///  Machine dependent class.
        class TMachineDependent
        {
            
          protected:
            
            ///  Different types of endianness.
            enum EEndianness
            {
              eLittleEndian , ///<  Little endian.
              eBigEndian      ///<  Big endian.
            };

            ///  Machine Endianness.
            EEndianness   eEndianType;


          public:
            
            //
            //  C O N S T R U C T O R S
            //
            
            ///  Builds a new instance.
            TMachineDependent (void);
            
            ///  Destroys the instance.
            virtual ~TMachineDependent (void) {}
            
            
          public:
            
            //
            //  S E L E C T O R S
            //
            
            /**
            *  Returns true if buffer is big-endian.
            *  @return True if buffer is big-endian.
            */
            bool isBigEndian (void) const
            {
              return ( eEndianType == eBigEndian );
            }
            
            /**
            *  Returns true if buffer is little-endian.
            *  @return True if buffer is little-endian.
            */
            bool isLittleEndian (void) const
            {
              return ( eEndianType == eLittleEndian );
            }
            
            /**
            *  Returns a long integer from binary buffer, no matter 
            *  what type of endian use this machine.
            *  @param eENDIAN_TYPE      Endian type.
            *  @param pkucSOURCE_BUFFER Source endianness dependent buffer.
            *  @param zLENGTH           Length of that buffer.
            *  @return A long integer result from converting the binary
            *          buffer.
            *  @see #EEndianness.
            */
            long int getEndianIndependentLongInt ( EEndianness          eENDIAN_TYPE      ,
                                                   const unsigned char* pkucSOURCE_BUFFER ,
                                                   std::size_t          zLENGTH           ) const;
            
        };  // class TMachineDependent
        
        ///  Base little endian buffer class for base operating system class.
        class TLittleEndianBuffer : private TMachineDependent
        {
      
          private:
      
            ///  Big endian buffer.
            const unsigned char*   pkucBuffer;
            
            ///  Buffer length in bytes.
            std::size_t            zLength;
            
      
          public:
            
            //
            //  C O N S T R U C T O R S
            //
            
            /**
            *  Builds the instance.
            *  @param pkucSOURCE_BUFFER Source buffer.
            *  @param zLENGTH           Length of the buffer.
            */
            TLittleEndianBuffer ( const unsigned char* pkucSOURCE_BUFFER ,
                                  std::size_t          zLENGTH           ) :
              pkucBuffer (pkucSOURCE_BUFFER)                               ,
              zLength    (zLENGTH)                                         {}
            
            
          public:
            
            //
            //  S E L E C T O R S
            //
            
            ///  Converts the buffer to long int.
            virtual operator long int (void) const
            {
              return getEndianIndependentLongInt (eLittleEndian, pkucBuffer, zLength);
            }

        };  // class TLittleEndianBuffer
        
        ///  Base big endian buffer class for base operating system class.
        class TBigEndianBuffer : private TMachineDependent
        {
      
          private:
            
            ///  Big endian buffer.
            const unsigned char*   pkucBuffer;
            
            ///  Buffer length in bytes.
            std::size_t            zLength;      
            
            
          public:
            
            //
            //  C O N S T R U C T O R S
            //
            
            /**
            *  Builds the instance.
            *  @param pkucSOURCE_BUFFER Source buffer.
            *  @param zLENGTH           Length of the buffer.
            */
            TBigEndianBuffer ( const unsigned char* pkucSOURCE_BUFFER ,
                               std::size_t          zLENGTH           ) :
              pkucBuffer (pkucSOURCE_BUFFER)                            ,
              zLength    (zLENGTH)                                      {}


          public:
      
            //
            //  S E L E C T O R S
            //
      
            ///  Converts the buffer to long int.
            virtual operator long int (void) const
            {
              return getEndianIndependentLongInt (eBigEndian, pkucBuffer, zLength);
            }
      
        };  // class TBigEndianBuffer


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Destroys the instance.
        virtual ~ISystem (void) {}
        

      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Get current directory.
        *  @return Current directory.
        */
        virtual TString getCurrentDirectory (void) const = 0;

        /**
        *  Get symbol for directory separator.
        *  @return The directory separator.
        */
        virtual char getDirectorySeparator (void) const = 0;

        /**
        *  Get file size.
        *  @param pkcFILE_NAME File name.
        *  @return Size of \a pkcFILE_NAME.
        */
        virtual std::size_t getFileSize (const char* pkcFILE_NAME) const = 0;

        /**
        *  Split a search path string  into  the path components.  The components
        *  must  be  separated  using the  symbol returned by getPathSeparator().
        *  Path items  with zero  length will  be omitted.  Returned  paths  will
        *  always be symbol terminated.
        *  @param rkySOURCE_PATH_COLLECTION String with symbol-separated paths.
        *  @return Vector with path items.
        *  @see #getPathSeparator().
        */
        virtual vector<TString> getPathItems (const string& rkySOURCE_PATH_COLLECTION) const;

        /**
        *  Get symbol for path separator.
        *  @return The path separator.
        */
        virtual char getPathSeparator (void) const = 0;

        /**
        *  Gets an unique integer value that identifies the current process.
        *  @return The process identifier.
        */
        virtual int getProcessId (void) const = 0;

        /**
        *  Sleeps the process or  thread \a zSECONDS  seconds and \a zNANOSECONDS
        *  nanoseconds.
        *  @param zSECONDS Number of seconds to sleep.
        *  @param zNANOSECONDS Number of nanoseconds to sleep.
        */
        virtual void sleep (std::size_t zSECONDS, std::size_t zNANOSECONDS = 0) const = 0;

        /**
        *  Sends a message to the system logger.
        *  @param rkyMESSAGE Message to log. It is a format, as in std::printf(3)
        *                    and  any  arguments required by  the format,  except
        *                    that  the two character %m  will  be replaced by the
        *                    error message string (std::strerror()) corresponding
        *                    to the present value of std::errno.
        *  @param eFACILITY  Log facility of the message.
        *  @param eLEVEL     Log level of the message.
        *  @see std::errno, std::printf(), std::strerror(), std::syslog().
        */
        virtual void writeLogEntry ( const string& rkyMESSAGE          ,
                                     ELogLevel     eLEVEL    = eNotice ,
                                     ELogFacility  eFACILITY = eUser   ) const = 0;

        /**
        *  Yields yield the processor.
        *  A process can relinquish the processor voluntarily without blocking by
        *  calling \a yield().  The process  will then be moved to the end of the
        *  queue for its static priority and a new process gets to run.
        *  @see std::sched_yield().
        */
        virtual void yield (void) const = 0;

    };  // class ISystem

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_SYSTEM__
