/*
*  Name:      pattern_based_code_generator.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Pattern based code generator	 
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_CODEGEN_PATTERN_BASED_CODE_GENERATOR__
#define _MPCL_TEXT_CODEGEN_PATTERN_BASED_CODE_GENERATOR__

#include "../../io/streamable.hh"
#include "../../io/streamable_map.hh"
#include "../string.hh"
#include "clause_body_string.hh"
#include "defs.hh"
#include "instruction_body.hh"
#include "instruction_tag_string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for CODEGEN.
    namespace codegen
    {

      using io::IStreamable;
      using io::TStreamableMap;

      /**
      *  Pattern-based code generator.
      *
      *  As its name says, its  function is, given a CDML file, an
      *  instruction  name,  and a clause name, it  generates  the
      *  corresponding code.
      *
      *  CDML (Code  Description  Mark-up Language) is a  language
      *  that defines a matter of generating parameterizable code.
      *  Each  CDML file can inherit from another CDML file  (just
      *  one), but this means that  only instructions/clauses that
      *  already stand in father CDML file, can be (re)defined.
      *  Variables can't contain other variable tags.
      *
      *  \warning The string "::" isn't permitted in clause names,
      *           because it is used for name mangling.
      */
      class TPatternBasedCodeGenerator: public IStreamable<>
      {

        private:

          ///  Instruction map type.
          typedef
            TStreamableMap<TInstructionTagString, TInstructionBody>
            TInstructionMap;

          ///  Map with instruction tag to instruction body associations.
          TInstructionMap   tInstructionMap;

          ///  Map with global runtime variables with its values (normal and file types).
          TStringToVariableMap   tVariableMap;

          ///  String with the CDML normal and inherited file path.
          TString   yCdmlPath;

          ///  String with the CDML file description.
          TString   yDescription;

          ///  String with the CDML-tag attribute named 'inherit'.
          TString   yInherit;

          ///  String with the !DOCTYPE-tag attribute named 'public'.
          TString   yPublic;

          ///  String with the !DOCTYPE-tag attribute named 'system'.
          TString   ySystem;

          ///  String with the CDML-tag attribute named 'target'.
          TString   yTarget;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance from a CDML file.
          *  @param rkySOURCE_CDML_FILE Source CDML file name.
          *  @param rkyCDML_PATH        This string musts be a colon separated
          *                             paths string, like "dir1/dir2:dir3/dir4".
          *                             Where symbol '/' must be changed for each
          *                             platform.
          */
          TPatternBasedCodeGenerator ( const TString& rkySOURCE_CDML_FILE ,
                                       const TString& rkyCDML_PATH        );

          ///  Builds a new instance without loading any CDML document.
          TPatternBasedCodeGenerator (void) :
            IStreamable<>   ()              ,
            tInstructionMap ()              ,
            tVariableMap    ()              ,
            yCdmlPath       ()              ,
            yDescription    ()              ,
            yInherit        ()              ,
            yPublic         ()              ,
            ySystem         ()              ,
            yTarget         ()
          {
            initConstants();
          }

          ///  Resets values removing all instructions.
          void clear (void)
          {
            tInstructionMap.clear();
            yTarget.erase();
            yInherit.erase();
            yDescription.erase();
            yCdmlPath.erase();
          }

          ///  Clears code from all clauses.
          void clearCode (void);

          /**
          *  Makes the CDML file to inherit from no other CDML file.
          *  It really only empties yInherit value and makes substitutions
          *  of entity '&eoc;'.
          */
          void compile (void);

          /**
          *  Loads a CDML file.
          *  @param rkySOURCE_CDML_FILE Source file name.
          *  @exception TExceptionPBCG_FileNotFound If file isn't found.
          */
          void loadFrom (const TString& rkySOURCE_CDML_FILE);

          /**
          *  Preprocess entities on variables.
          */
          void preprocessVariables (void)
          {
            TStringToVariableMap::iterator         I             = tVariableMap.begin();
            TStringToVariableMap::const_iterator   ktVariableEnd = tVariableMap.end();

            for (; ( I != ktVariableEnd ) ;++I)
            {
              InstantiateEntities (I->second);
            }
          }

          /**
          *  Updates attribute \a yDescription.
          *  @param pkcSOURCE_DESCRIPTION Source description string.
          */
          void setDescription (const char* pkcSOURCE_DESCRIPTION)
          {
            yDescription = pkcSOURCE_DESCRIPTION;
          }

          /**
          *  Sets path for loading CDML files.
          *  @param rkyCDML_PATH This string must be a colon separated
          *                      paths string, like "dir1/dir2:dir3/dir4".
          *                      Where '/' must be changed for every platform.
          */
          void setCdmlPath (const TString& rkyCDML_PATH)
          {
            yCdmlPath = rkyCDML_PATH;
          }


        protected:

          //
          //  C O N S T R U C T O R S
          //

          ///  Initializes pattern-length constants if needed.
          void initConstants (void) const;

          /**
          *  Inherits from another CDML file.
          *  @param rkySOURCE_CDML_FILE Source CDML file name.
          *  @warning If the file \a rkySOURCE_CDML_FILE is the same as
          *           ours, a loop will happen.
          */
          void inheritFrom (const TString& rkySOURCE_CDML_FILE);

          /**
          *  Loads the instance from stream \a rtSOURCE_STREAM.
          *  @param rtSOURCE_STREAM Source stream.
          */
          void read (std::basic_istream<char_type, traits_type>& rtSOURCE_STREAM);


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Gets the variable map.
          *  @return The variable map.
          *  @see TStringToVariableMap.
          */
          const TStringToVariableMap& variables (void) const
          {
            return tVariableMap;
          }

          const TClauseBodyString& codePatternFor ( const char* pkcINSTRUCTION_NAME ,
                                                    const char* pkcCLAUSE_NAME      ) const;

          TInstructionBody& instruction (const char* pkcINSTRUCTION_NAME);

          const TInstructionBody& instruction (const char* pkcINSTRUCTION_NAME) const;

          /**
          *  Returns true if there is a variable named \a pkcVARIABLE_NAME.
          *  @param pkcVARIABLE_NAME Name of the variable.
          *  @return True if there is a variable named \a pkcVARIABLE_NAME.
          */
          bool existsVariable (const char* pkcVARIABLE_NAME) const;

          bool existsClause (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME) const;

          /**
          *  Gets the instruction body corresponding to instruction
          *  \a pkcINSTRUCTION_NAME. This member function is a synonym
          *  of instruction().
          *  @param pkcINSTRUCTION_NAME Instruction name.
          *  @return The instruction body.
          */
          const TInstructionBody& operator [] (const char* pkcINSTRUCTION_NAME) const
          {
            return instruction (pkcINSTRUCTION_NAME);
          }

          /**
          *  Gets the instruction body corresponding to instruction
          *  \a pkcINSTRUCTION_NAME. This member function is a synonym
          *  of instruction().
          *  @param pkcINSTRUCTION_NAME Instruction name.
          *  @return The instruction body.
          */
          TInstructionBody& operator [] (const char* pkcINSTRUCTION_NAME)
          {
            return instruction (pkcINSTRUCTION_NAME);
          }


        protected:

          //
          //  S E L E C T O R S
          //

          /**
          *  Writes the instance onto stream \a rtTARGET_OSTREAM.
          *  @param rtTARGET_OSTREAM Target output stream.
          */
          void write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const;

      };  // class TPatternBasedCodeGenerator

    }  // namespace codegen

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_CODEGEN_PATTERN_BASED_CODE_GENERATOR__
