/*
*  Name:         date.hh
*  Author:       Francisco Rodrigo Escobedo Robles
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Base date class
*  Date:         $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:     $Revision: 9838 $
*
*  Copyright (C) 2000-2001  Francisco Rodrigo Escobedo Robles <frer@pepix.net>
*  Copyright (C) 2001-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_DATE__
#define _MPCL_TEXT_DATE__

#include <algorithm>
#include "../exceptions.hh"
#include "string.hh"


///  Name-space for MPCL.
namespace mpcl
{
  
  ///  Name-space for TEXT.
  namespace text
  {

    ///  Represents a time difference in seconds.
    class TTimeInterval
    {

      private:

        ///  Internal interval representation in seconds.
        double dInterval;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TTimeInterval (void) :
  	dInterval (0.0)      {}

        ///  Builds a new instance with an initial value.
        TTimeInterval (double dINTERVAL) :
  	dInterval (dINTERVAL)            {}


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Returns the seconds in the interval.
        *  @return Seconds in the interval.
        */
        double seconds (void) { return dInterval; }

        /**
        *  Returns the minutes in the interval.
        *  @return Minutes in the interval.
        */
        double minutes (void) { return dInterval / 60; }

        /**
        *  Returns the hours in the interval.
        *  @return Hours in the interval.
        */
        double hours (void) { return dInterval / 3600; }

        /**
        *  Returns the days in the interval.
        *  @return Days in the interval.
        */
        double days (void) { return dInterval / 86400; }

    };  // class TTimeInterval


    ///  BaseDate abstract data class.
    class IDate
    {

      protected:

        ///  Constants.
        static const int   _kiMaxFormattedDate = 517;

        ///  Total number of seconds from the Epoch.
        time_t   tSecondsFromEpoch;


      public:

        ///  Date types.
        enum EDateType { eEmpty, eNow, eTomorrow };


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds a new instance.  The base date must be supplied but the hour is
        *  optional.
        *  @param kiYEAR    Year.
        *  @param kiMONTH   Month.
        *  @param kiDAY     Day.
        *  @param kiHOUR    Hour.
        *  @param kiMINUTES Minutes.
        *  @param kiSECONDS Seconds.
        */
        IDate ( const int kiYEAR        ,
                const int kiMONTH       ,
                const int kiDAY         ,
                const int kiHOUR    = 0 ,
                const int kiMINUTES = 0 ,
                const int kiSECONDS = 0 ) :
          tSecondsFromEpoch (-1)
        {
          set (kiYEAR, kiMONTH, kiDAY, kiHOUR, kiMINUTES, kiSECONDS);
        }

        /**
        *  Builds a new instance.
        *  @param keDATE_TYPE Date type.
        */
        IDate (const EDateType keDATE_TYPE = eNow)
          throw (TConstraintException);

        /**
        *  Adds a specified number of seconds.
        *  @param kiSECONDS Seconds to be added.
        */
        IDate& addSeconds (const int kiSECONDS)
          throw (TConstraintException)
        {
          if ( !empty() )
          {
            tSecondsFromEpoch += kiSECONDS;
          }
          else
          {
            throw TConstraintException ("invalid date operation", __FILE__, __LINE__);
          }
          return *this;
        }

        /**
        *  Adds a specified number of minutes.
        *  @param kiMINUTES Minutes to be added.
        */
        IDate& addMinutes (const int kiMINUTES)
        {
          return addSeconds (kiMINUTES * 60);
        }

        /**
        *  Adds a specified number of hours.
        *  @param kiHOURS Hours to be added.
        */
        IDate& addHours (const int kiHOURS)
        {
          return addSeconds (kiHOURS * 3600);
        }

        /**
        *  Adds a specified number of days.
        *  @param kiDAYS Days to be added.
        */
        IDate& addDays (const int kiDAYS)
        {
          return addSeconds (kiDAYS * 86400);
        }

        /**
        *  Sets a new date.
        *  @param kiYEAR    Year.
        *  @param kiMONTH   Month.
        *  @param kiDAY     Day.
        *  @param kiHOUR    Hour.
        *  @param kiMINUTES Minutes.
        *  @param kiSECONDS Seconds.
        *  @exception TConstraintException When the date isn't valid.
        */
        void set ( const int kiYEAR        ,
                   const int kiMONTH       ,
                   const int kiDAY         ,
                   const int kiHOUR    = 0 ,
                   const int kiMINUTES = 0 ,
                   const int kiSECONDS = 0 )
          throw (TConstraintException);


      public:

        //
        //  S E L E C T O R S
        //

        ///  Tells if a date is empty.
        bool empty (void) const
        {
          return ( (time_t) -1 == tSecondsFromEpoch );
        }

        /**
        *  Gets the current date in an specified strftime-like format.
        *  @see std::strftime().
        */
        virtual TString get (const char* pkcDATE_FORMAT) const = 0;

        /**
        *  Gets the current date in an specified strftime-like format.
        *  @see std::strftime().
        */
        TString get (const TString& rkyDATE_FORMAT) const
        {
          return get (rkyDATE_FORMAT.c_str());
        }

        /**
        *  Returns true  if  this  instance  is  between  \a rktBEGIN_DATE  and
        *  \a rktEND_DATE dates.
        *  @param rktBEGIN_DATE Beginning date.
        *  @param rktEND_DATE   Ending date.
        *  @return True if current date is between beginning and ending dates.
        */
        bool in ( const IDate& rktBEGIN_DATE ,
                  const IDate& rktEND_DATE   ) const
        {
          return ( ( *this >= rktBEGIN_DATE ) && ( *this <= rktEND_DATE ) );
        }

        /**
        *  Returns true if two dates are equal.
        *  @return True if two dates are equal.
        *  @exception TConstraintException If at least one  of the exceptions
        *                                  is empty.
        */
        bool operator == (const IDate& rktDATE) const
          throw (TConstraintException)
        {
          if ( empty() || rktDATE.empty() )
          {
            throw TConstraintException ("invalid date comparison", __FILE__, __LINE__);
          }
          return ( tSecondsFromEpoch == rktDATE.tSecondsFromEpoch );
        }

        bool operator != (const IDate& rktDATE) const
        {
          return !( *this == rktDATE );
        }

        /**
        *  Returns true if second dates are previous than the first.
        *  @return True if second dates are previous than the first.
        *  @exception TConstraintException If at least one  of the exceptions
        *                                  is empty.
        */
        bool operator < (const IDate& rktDATE) const
          throw (TConstraintException)
        {
          if ( empty() || rktDATE.empty() )
          {
            throw TConstraintException ("invalid date comparison", __FILE__, __LINE__);
          }
          return ( tSecondsFromEpoch < rktDATE.tSecondsFromEpoch );
        }

        bool operator > (const IDate& rktDATE) const
        {
          return ( rktDATE < *this );
        }

        bool operator <= (const IDate& rktDATE) const
        {
          return !( rktDATE < *this );
        }

        bool operator >= (const IDate& rktDATE) const
        {
          return !( *this < rktDATE );
        }

        /**
        *  Gets the difference in seconds between two \a IDate objects.
        *  @param rktDATE The date to substract.
        */
        TTimeInterval operator - (const IDate& rktDATE) const
        {
          return TTimeInterval (difftime (tSecondsFromEpoch, rktDATE.tSecondsFromEpoch));
        }

    };  // class IDate

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_DATE__
