/*
*  Name:      container.hh
*  Author:    Rafael Jesus Alcantara Perez
*             Angel Jimenez Jimenez
*  Summary:   HTML container
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1994-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_CONTAINER__
#define _MPCL_TEXT_HTML_CONTAINER__

#include "../../util/collection/map.hh"
#include "../../util/general-functions.hh"
#include "tag.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      using util::collection::TMap;

      ///  Forward declaration.
      class TContainer;

      ///  Smart pointer to TContainer.
      typedef
        TSmartPointer<TContainer>
        QTContainer;

      ///  Smart pointer to const TContainer.
      typedef
        TSmartPointer<const TContainer>
        QKTContainer;

      /**
      *  HTML container base class.
      *
      *  When you insert an  anonymous  tag  (without  identifier),  it is given a
      *  random  identifier  (for  the container map, the tag  identifier  remains
      *  untouched). If you insert two times the same tag, it is given an extended
      *  identifier (for the container map only).
      *
      *  When a tag is inserted and its name is already in the container, then its
      *  name is mangled using character '_'.
      *
      *  Container  instances  that  were  trying to supplant  another  tag,  must
      *  overload the next member functions:
      *
      *    operator [](), identifier(), value() and setValue().
      *
      *  Requirements:  All  tags  contained  have to be allocated  with  the  new
      *                 operator, because  when  the container is  destroyed,  all
      *                 tags contained are deleted.
      */
      class TContainer : public TTag
      {

        public:

          ///  Vector type with smart-pointers to tags.
          typedef
            TMap<TString, QTTag>
            TStringToTagMap;


        protected:

          ///  List with smart-pointers (to HTML tags).
          TTagList   tTagList;

          /**
          *  Map  with  tag pseudo-identifiers to smart-pointers (to  HTML  tags).
          *  This  class  hasn't  any order, but it must exist  to  enable  random
          *  access.
          */
          mutable TStringToTagMap   tStringToTagMap;


        public:

          //
          //  C O N S T R U C T O R S
          //

          TContainer (void)    :
            TTag            () ,
            tTagList        () ,
            tStringToTagMap ()
          {
            tAttributeMap ["class-identifier"] = "container";
            tAttributeMap.bind ("group-align", "misaligned");
          }

          TContainer ( const char* pkcIDENTIFIER                 ,
                       const char* pkcVALUE       = ""           ,
                       const char* pkcGROUP_ALIGN = "misaligned" ) :
            TTag            (pkcIDENTIFIER, pkcVALUE)              ,
            tTagList        ()                                     ,
            tStringToTagMap ()
          {
            tAttributeMap ["class-identifier"] = "container";
            tAttributeMap.bind ("group-align", pkcGROUP_ALIGN);
          }

          void clear (void)
          {
            tTagList.clear();
            tStringToTagMap.clear();
          }

          /**
          *  Tries to  find a tag named  \a rkyIDENTIFIER  from the tags stored (if
          *  any)  in this instance,  within  first class.  If the tag is not found
          *  then it is inserted.
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          */
          virtual QTTag findOrInsertTag (const TString& rkyIDENTIFIER);

          /**
          *  Inserts   the   pointer \a rkqtTAG  into  this  container,  with  name
          *  \a rkyTAG_IDENTIFIER.
          *  @param rkyTAG_IDENTIFIER Tag identifier.
          *  @param rkqtTAG           Tag to be inserted.
          *  @param tINDEX            Iterator  pointing to the  position where the
          *                           tag rqtTAG must be inserted.
          */
          void insertAs ( const TString& rkyTAG_IDENTIFIER ,
                          const QTTag&   rkqtTAG           ,
                          iterator       tINDEX            );

          /**
          *  Inserts the  smart pointer  \a rkqtTAG into this  container.  This
          *  member  method  is used  to  update  the  source  tag  pointed  by
          *  \a rkqtTAG and then, to insert that tag into this container.
          *  @param rkqtTAG Tag to be inserted.
          */
          virtual void insertUpdating (const QTTag& rkqtTAG)
          {
            insertAs (rkqtTAG->identifier(), rkqtTAG, end());
          }

          /**
          *  Inserts the  smart pointer \a rkqtTAG  into this  container.  This
          *  member  method is  used  to  update  the  source  tag  pointed  by
          *  \a rkqtTAG and then, to insert that tag into this container.
          *  @param rkqtTAG Tag to be inserted.
          *  @param tINDEX  Iterator at the position where the tag must be inserted.
          */
          virtual void insertUpdating (const QTTag& rkqtTAG, iterator tINDEX)
          {
            insertAs (rkqtTAG->identifier(), rkqtTAG, tINDEX);
          }

          /**
          *  Inserts smart pointer \a rkqtTAG into this container.
          *  @param rkqtTAG Tag to be inserted.
          */
          virtual void insert (const QTTag& rkqtTAG)
          {
            insertUpdating (rkqtTAG);
          }

          /**
          *  Inserts smart pointer \a rkqtTAG into this container.
          *  @param rkqtTAG Tag to be inserted.
          *  @param tINDEX  Iterator at the position where the tag must be inserted.
          */
          virtual void insert (const QTTag& rkqtTAG, iterator tINDEX)
          {
            insertUpdating (rkqtTAG, tINDEX);
          }

          void reset (void)
          {
            iterator   I    = tTagList.begin();
            iterator   tEnd = tTagList.end();

            TTag::reset();
            for (; ( I != tEnd ) ;++I)
            {
              (*I)->reset();
            }
          }

          /**
          *  Sets CSS class.
          *  @param rkyCLASS  CSS class name.
          *  @param gDELEGATE If true,  the  class  of inner tags  are also  set to
          *                   class \a rkyCLASS.
          */
          void setClass (const TString& rkyCLASS, bool gDELEGATE = false);

          /**
          *  Sets CSS class.
          *  @param pkcCLASS  CSS class name.
          *  @param gDELEGATE If true,  the class  of inner  tags  are also  set to
          *                   class \a pkcCLASS.
          */
          void setClass (const char* pkcCLASS, bool gDELEGATE = false);

          /**
          *  Assigns a value to the attribute "id" in the attribute map.
          *  @param pkcID     New value of the attribute "id".
          *  @param gDELEGATE Make objects contained to have the same id.
          */
          void setId (const char* pkcID, bool gDELEGATE = false);

          /**
          *  Assigns a value to the attribute "id" in the attribute map.
          *  @param rkyID New value of the attribute "id".
          *  @param gDELEGATE Make objects contained to have the same id.
          */
          void setId (const TString& rkyID, bool gDELEGATE = false);

          void update (void)
          {
            iterator   I    = tTagList.begin();
            iterator   tEnd = tTagList.end();

            for (; ( I != tEnd ) ;++I)
            {
              (*I)->update();
            }
          }


        public:

          //
          //  S E L E C T O R S
          //

          iterator begin (void)
          {
            return tTagList.begin();
          }

          const_iterator begin (void) const
          {
            return tTagList.begin();
          }

          iterator end (void)
          {
            return tTagList.end();
          }

          const_iterator end (void) const
          {
            return tTagList.end();
          }

          /**
          *  Tries to  find a tag named  \a rkyIDENTIFIER  from the tags stored (if
          *  any) in this instance, within  first class.
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return Returns that tag or NULL if not.
          */
          virtual QTTag findTag (const TString& rkyIDENTIFIER) const;

          /**
          *  Returns true if there  is  an tag named \a rkyIDENTIFIER  between  the
          *  tags stored (if any) in this instance, within first class.
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return True if there is that tag.
          */
          virtual bool hasTag (const TString& rkyIDENTIFIER) const
          {
            return !findTag (rkyIDENTIFIER).isNull();
          }

          QTTag tag (const TString& rkyIDENTIFIER) const;

          /**
          *  Gets  the tag named \a pkcIDENTIFIER from the tags stored (if any)  in
          *  this instance,  within first class.  It is a synonym of tag().
          *  @param pkcIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          */
          const TTag& operator [] (const char* pkcIDENTIFIER) const
          {
            return *tag (pkcIDENTIFIER);
          }

          /**
          *  Gets  the  tag named \a rkyIDENTIFIER from the tags stored (if any) in
          *  this instance, within first class. It is a synonym of tag().
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          */
          const TTag& operator [] (const TString& rkyIDENTIFIER) const
          {
            return *tag (rkyIDENTIFIER);
          }

          /**
          *  Gets  the  tag named \a pkcIDENTIFIER from the tags stored (if any) in
          *  this instance, within first class. It is a synonym of tag().
          *  @param pkcIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          */
          TTag& operator [] (const char* pkcIDENTIFIER)
          {
            return *tag (pkcIDENTIFIER);
          }

          /**
          *  Gets  the tag named \a rkyIDENTIFIER from the tags stored  (if any) in
          *  this instance, within first class. It is a synonym of tag().
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          */
          TTag& operator [] (const TString& rkyIDENTIFIER)
          {
            return *tag (rkyIDENTIFIER);
          }

          /**
          *  Gets the number of tags stored in this instance, within second class.
          *
          *  \warning  Second class tags may be the same to first class tags.
          */
          size_type size (void) const
          {
            return tTagList.size();
          }


        protected:

          //
          //  S E L E C T O R S
          //

          void writeUsing ( std::basic_ostream<char>& rtTARGET_OSTREAM        ,
                            const TString&            rkyQUALIFIED_IDENTIFIER ) const;

      };  // class TContainer

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_CONTAINER__
