/*
*  Name:      tag.hh
*  Author:    Angel Jimenez Jimenez
*             Rafael Jesus Alcantara Perez
*  Summary:   HTML base tag
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1994-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_TAG__
#define _MPCL_TEXT_HTML_TAG__

#include <list>
#include <memory>
#include "../../io/output_streamable.hh"
#include "../../memory/smart_pointer.hh"
#include "../../util/collection/string_to_string_map.hh"
#include "../codegen/pattern_based_code_generator.hh"
#include "../string.hh"
#include "defs.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      using memory::TSmartPointer;
      using text::codegen::TPatternBasedCodeGenerator;
      using util::collection::TStringToStringMap;

      ///  Forward declaration.
      class TTag;

      ///  Smart pointer to TTag.
      typedef
        TSmartPointer<TTag>
        QTTag;

      ///  Smart pointer to const TTag.
      typedef
        TSmartPointer<const TTag>
        QKTTag;

      /**
      *  Base class for HTML tags.
      *
      *  This  class doesn't represent exactly the HTML tags elements, rather than
      *  this, they represent somehow, a meta-tag that can represent a set of HTML
      *  tags (though this set cardinal can be one).
      *  There are two classes for store-able  tags. The first class is for normal
      *  tags, and the second class is for special purpose tags.
      *
      *  Attributes (accessed through member method attribute()):
      *
      *    "class-identifier"     ::= Class identifier (used  as canonical name in
      *                               \a _ptCodeGenerator).
      *    "qualified-identifier" ::= Tag identifier (used as a way of identifying
      *                               the tag through the containers tree).
      *    "identifier"           ::= *.NAME.
      *    "value"                ::= *.VALUE.
      *
      *  \warning Only tags initialized with identifiers, will be assignable.
      */
      class TTag : public IOutputStreamable<>
      {

        public:

          ///  List type with smart-pointers to tags.
          typedef
            std::list<QTTag>
            TTagList;

          ///  Constant iterator for the tag list.
          typedef
            TTagList::const_iterator
            const_iterator;

          ///  Iterator for the tag list.
          typedef
            TTagList::iterator
            iterator;

          ///  Size type for the tag list.
          typedef
            TTagList::size_type
            size_type;


        protected:

          ///  Static pointer to a code-generator.
          static TPatternBasedCodeGenerator*   _ptCodeGenerator;

          ///  True if identifier isn't forwarded to inner tags.
          bool   gForwardIdentifier;

          ///  True if tag has been dynamically created.
          bool   gHasBeenDynamiclyCreated;

          ///  True if tag can be dynamicly loaded.
          bool   gIsDynamiclyLoadable;

          /**
          *  Boolean with truth value iif  identifier  has to be included in inner
          *  tags ("qualified-identifier").
          */
          bool   gIsQualified;

          /**
          *  Boolean with  truth  value  iif  identifier  does  not transmit  the
          *  attribute "qualified-identifier" to inner tags.
          */
          bool   gIsRootNode;

          ///  Boolean with truth value iif tag is visible (must be written).
          bool   gIsVisible;

          ///  Map with attribute name to value correspondences.
          TStringToStringMap   tAttributeMap;


        public:

          //
          //  L O C A L   F U N C T I O N S
          //

          ///  Sets code generator for all instances.
          static void _setCodeGenerator (const TPatternBasedCodeGenerator& rktCG);


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance.
          *  @param gDYNAMICALLY_CREATED True if it's dynamically created.
          */
          TTag (bool gDYNAMICALLY_CREATED = false);

          /**
          *  Builds a new assignable instance.
          *  @param pkcIDENTIFIER Identifier.
          *  @param pkcVALUE      Value.
          */
          TTag (const char* pkcIDENTIFIER, const char* pkcVALUE = "");

          /**
          *  Builds a new assignable instance.
          *  @param pkcIDENTIFIER        Identifier.
          *  @param gDYNAMICALLY_CREATED True if it's dynamically created.
          */
          TTag (const char* pkcIDENTIFIER, bool gDYNAMICALLY_CREATED);

          ///  Destroys the instance.
          virtual ~TTag (void);

          ///  Clears instance.
          virtual void clear (void) {}

          ///  Clears attribute "value".
          virtual void reset (void)
          {
            //
            //  If this is an assignable tag.
            //
            if ( isAssignable() )
            {
              tAttributeMap ["value"].erase();
            }
          }

          /**
          *  Sets CSS class.
          *  @param rkyCLASS  CSS class name.
          *  @param gDELEGATE Make objects contained to belong to the same class.
          */
          virtual void setClass ( const TString& rkyCLASS          ,
                                  bool           gDELEGATE = false );

          /**
          *  Sets CSS class.
          *  @param pkcCLASS  CSS class name.
          *  @param gDELEGATE Make objects contained to belong to the same class.
          */
          virtual void setClass ( const char* pkcCLASS          ,
                                  bool        gDELEGATE = false );

          /**
          *  Sets dynamic load ability.
          *  @param gDYNAMICLY_LOADABLE True if the tag is dynamicly loadable.
          */
          void setDynamiclyLoadable (bool gDYNAMICLY_LOADABLE)
          {
            gIsDynamiclyLoadable = gDYNAMICLY_LOADABLE;
          }

          /**
          *  Assigns a value to the attribute "id" in the attribute map.
          *  @param pkcID     New value of the attribute "id".
          *  @param gDELEGATE Make objects contained to have the same id.
          */
          virtual void setId (const char* pkcID, bool = false)
          {
            attribute ("id") = pkcID;
          }

          /**
          *  Assigns a value to the attribute "id" in the attribute map.
          *  @param rkyID New value of the attribute "id".
          *  @param gDELEGATE Make objects contained to have the same id.
          */
          virtual void setId (const TString& rkyID, bool = false)
          {
            attribute ("id") = rkyID;
          }

          /**
          *  Assings a value to the attribute "value" in the attribute map.
          *  @param rkyVALUE Value to assign.
          */
          virtual void setValue (const TString& rkyVALUE)
          {
            setValue (rkyVALUE.c_str());
          }

          /**
          *  Assings a value to the attribute "value" in the attribute map.
          *  @param pkcVALUE Value to assign.
          */
          virtual void setValue (const char* pkcVALUE)
          {
            if ( !isAssignable() )
            {
              throw TConstraintException ("instance is not assignable", __FILE__, __LINE__);
            }
            tAttributeMap ["value"] = pkcVALUE;
          }

          /**
          *  Sets visibility of the tag.
          *  @param gIS_VISIBLE True if the tag must be written.
          */
          void setVisibility (bool gIS_VISIBLE)
          {
            gIsVisible = gIS_VISIBLE;
          }

          /**
          *  Updates the object. This member function is used by HTML containers to
          *  update  the  value of the  object,  before writing the object  onto  a
          *  stream.
          */
          virtual void update (void) {}


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Gets attribute named \a pkcNAME.
          *  @param pkcNAME Source attribute name.
          */
          virtual TString& attribute (const char* pkcNAME)
          {
            return tAttributeMap [pkcNAME];
          }

          /**
          *  Gets attribute named \a pkcNAME.
          *  @param pkcNAME Source attribute name.
          */
          virtual const TString& attribute (const char* pkcNAME) const
          {
            return tAttributeMap [pkcNAME];
          }

          virtual iterator begin (void);

          virtual const_iterator begin (void) const;

          virtual iterator end (void);

          virtual const_iterator end (void) const;

          /**
          *  Gets CSS class.
          *  @return CSS class name.
          */
          TString getClass (void) const
          {
            return tAttributeMap ["class"];
          }

          /**
          *  Gets attribute ID.
          *  @return Attribute ID.
          */
          const TString& getId (void) const
          {
            return tAttributeMap ["id"];
          }

          ///  Returns the value of the "value" attribute.
          virtual const TString& getValue (void) const
          {
            return tAttributeMap ["value"];
          }

          /**
          *  Returns true  if there is an  attribute  named \a pkcNAME  between the
          *  tags stored (if any) in this instance, within first class.
          *  @param pkcNAME Source attribute name.
          */
          virtual bool hasAttribute (const char* pkcNAME) const
          {
            return ( tAttributeMap.end() != tAttributeMap.find (pkcNAME) );
          }

          ///  Tells if this is a dynamically loadable instance.
          bool hasBeenDynamiclyCreated (void) const
          {
            return gHasBeenDynamiclyCreated;
          }

          ///  Returns the value of the "identifier" attribute.
          virtual const TString& identifier (void) const
          {
            return tAttributeMap ["identifier"];
          }

          ///  Tells if this is an assignable instance.
          virtual bool isAssignable (void) const
          {
            return !identifier().empty();
          }

          ///  Tells if this is a dynamically loadable instance.
          bool isDynamiclyLoadable (void) const
          {
            return gIsDynamiclyLoadable;
          }

          ///  Tells if this is an assignable instance.
          bool isQualified (void) const
          {
            return gIsQualified;
          }

          /**
          *  Gets visibility of the tag.
          *  @return True if the tag is visible.
          */
          virtual bool isVisible (void) const
          {
            return gIsVisible;
          }

          virtual bool operator == (const TTag& rktTAG) const
          {
            return ( identifier() == rktTAG.identifier() );
          }

          /**
          *  Gets  the tag named \a rkyIDENTIFIER from the tags stored  (if any) in
          *  this instance, within first class.
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          */
          virtual QTTag tag (const TString& rkyIDENTIFIER) const;

          /**
          *  Gets attribute named \a pkcNAME. It is a synonym of
          *  attribute().
          *  @param pkcNAME Source attribute name.
          */
          virtual TString& operator () (const char* pkcNAME)
          {
            return attribute (pkcNAME);
          }

          /**
          *  Gets   attribute   named   \a pkcNAME.  It  is  a  synonym  of
          *  attribute().
          *  @param pkcNAME Source attribute name.
          */
          virtual const TString& operator () (const char* pkcNAME) const
          {
            return attribute (pkcNAME);
          }

          /**
          *  Gets the tag named  \a pkcIDENTIFIER  from the tags stored (if any) in
          *  this instance, within first class. It is a synonym of tag().
          *  @param pkcIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          *  @exception TConstraintException Always, because this class is not a
          *                                  container.
          *  \warning This operator is implemented here to ease the access to the
          *           container instances (tHtmlPage ["mytable"]["myform"]["myinput"]).
          */
          virtual const TTag& operator [] (const char* pkcIDENTIFIER) const;

          ///  Synonym of the previous member function.
          virtual const TTag& operator [] (const TString& rkyIDENTIFIER) const
          {
            return operator [] (rkyIDENTIFIER.c_str());
          }

          /**
          *  Gets  the tag named \a pkcIDENTIFIER from the tags stored  (if any) in
          *  this instance, within first class. It is a synonym of tag().
          *  @param pkcIDENTIFIER Source tag identifier.
          *  @return Returns that tag.
          *  @exception TConstraintException Always, because this class is not a
          *                                  container.
          *  \warning This operator is implemented here to ease the access to the
          *           container instances (tHtmlPage ["mytable"]["myform"]["myinput"]).
          */
          virtual TTag& operator [] (const char* pkcIDENTIFIER);

          ///  Synonym of the previous member function.
          virtual TTag& operator [] (const TString& rkyIDENTIFIER)
          {
            return operator [] (rkyIDENTIFIER.c_str());
          }

          ///  Returns the value of the "qualified-identifier" attribute.
          virtual const TString& qualifiedIdentifier (void) const
          {
            return tAttributeMap ["qualified-identifier"];
          }

          /**
          *  Gets the number of tags stored in this instance.
          *  @return Tags cardinal.
          *  @exception TConstraintException Always, because this class is not a
          *                                  container.
          *  \warning This member function is implemented here to ease the access
          *           to the container instances (tHtmlPage ["mytable"].size()).
          */
          virtual size_type size (void) const;

          void write (std::basic_ostream<char>& rtOUTPUT_OSTREAM) const
          {
            writeUsing (rtOUTPUT_OSTREAM, "");
          }

          /**
          *  Writes  on  \a rtTARGET_OSTREAM  this i nstance,  using the identifier
          *  \a rkySOURCE_QUALIFIED_IDENTIFIER plus qualified identifier.
          *  @param rtTARGET_OSTREAM               Target stream.
          *  @param rkySOURCE_QUALIFIED_IDENTIFIER Source qualified identifier.
          *  @see #qualifiedIdentifier().
          */
          virtual void
          writeUsing ( std::basic_ostream<char>& rtTARGET_OSTREAM               ,
                       const TString&            rkySOURCE_QUALIFIED_IDENTIFIER ) const;


        protected:

          //
          //  S E L E C T O R S
          //

          TString code (const char* pkcINSTRUCTION, const char* pkcCLAUSE) const;

          TString code ( const char*               pkcINSTRUCTION ,
                         const char*               pkcCLAUSE      ,
                         const TStringToStringMap& rktVALUE_MAP   ) const;

          void expand (TStringToStringMap& rtTARGET_VALUE_MAP) const;

      };  // class TTag

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_TAG__
