/*
*  Name:      matcher.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Regular expression matcher
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1994-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_REGEX_MATCHER__
#define _MPCL_TEXT_REGEX_MATCHER__

#include <cstring>
#include <sstream>
#include "../../util/collection/string_to_string_map.hh"
#include "../string.hh"
#include "defs.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for REGEX.
    namespace regex
    {

      using util::collection::TStringToStringMap;

      /**
      *  Very simple regular expression matcher class.
      *
      *  Recognized patterns:
      *
      *    [[:space:]]+ ::=  {[[:space:]]}
      *    [[:space:]]* ::=  [{[[:space:]]}]
      *    [[:space:]]  ::=  c         / ( isspace (c) == true )
      *    [[:blank:]]+ ::=  {[[:blank:]]}
      *    [[:blank:]]* ::=  [{[[:blank:]]}]
      *    [[:blank:]]  ::=  c         / ( ( c == ' ' ) || ( c == '\t' ) )
      *    [.]*         ::=  c*        / text between tags
      *    c?           ::=  [c]
      *    %d           ::=  d+        / ( isdigit (c) == true )
      *    %s           ::=  c+        / ( ( c == '_' )            ||
      *                                    ( c == '-' )            ||
      *                                    ( isalnum (c) == true )  )
      *    %q           ::=  "d+" | c+ / ( ( d != '"' ) && ( ( c == '_' )            ||
      *                                                      ( c == '-' )            ||
      *                                                      ( isalnum (c) == true )  )
      *    %t           ::=  c*        / text between tags
      *    <<EOF>>      ::=  End of input
      */
      class TMatcher
      {

        private:

          ///  True when it is case-sensitive (true).
          bool   gCaseSensitive;

          ///  True when the stream has been created locally.
          bool   gLocalStream;

          ///  Source stream.
          std::basic_istream<char>*   ptSourceIstream;

          ///  Map with terminal definitions.
          TStringToStringMap   tTerminalMap;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TMatcher (void)           :
            gCaseSensitive  (true)  ,
            gLocalStream    (false) ,
            ptSourceIstream (NULL)  ,
            tTerminalMap    ()      {}

          /**
          *  Builds a new instance.
          *  @param rtISTREAM Source stream for loading.
          */
          TMatcher (std::basic_istream<char>& rtISTREAM) :
            gCaseSensitive  (true)                       ,
            gLocalStream    (false)                      ,
            ptSourceIstream (&rtISTREAM)                 ,
            tTerminalMap    ()
          {
            checkStream();
          }

          /**
          *  Builds a new instance.
          *  @param rySTRING Source stream for loading.
          */
          TMatcher (const std::string& rySTRING)                            :
            gCaseSensitive  (true)                                          ,
            gLocalStream    (true)                                          ,
            ptSourceIstream (new std::basic_istringstream<char> (rySTRING)) ,
            tTerminalMap    ()
          {
            if ( !rySTRING.empty() )
            {
              checkStream();
            }
          }

          /**
          *  Builds a new instance.
          *  @param pkcSTRING Source string for loading.
          */
          TMatcher (const char* pkcSTRING)                                   :
            gCaseSensitive  (true)                                           ,
            gLocalStream    (true)                                           ,
            ptSourceIstream (new std::basic_istringstream<char> (pkcSTRING)) ,
            tTerminalMap    ()
          {
            if ( pkcSTRING && std::strlen (pkcSTRING) )
            {
              checkStream();
            }
          }

          ///  Destroys the instance.
          ~TMatcher (void)
          {
            clearStream();
          }

          ///  Clears all terminal definitions.
          void clearDefinitions (void);

          /**
          *  Defines a terminal with name \a pkcTERMINAL, and definition
          *  \a pkcDEFINITION.
          *  @param pkcTERMINAL   Terminal to be defined.
          *  @param pkcDEFINITION Definition of the terminal.
          */
          void define (const char* pkcTERMINAL, const char* pkcDEFINITION);

          /**
          *  Redefines a terminal with name \a pkcTERMINAL, and definition
          *  \a pkcDEFINITION.
          *  @param pkcTERMINAL   Terminal to be redefined.
          *  @param pkcDEFINITION Definition of the terminal.
          */
          void redefine (const char* pkcTERMINAL, const char* pkcDEFINITION);

          /**
          *  Sets case-sensitiveness state.
          *  @param gTRUTH True if it will be case sensitive.
          */
          void setCaseSensitiveness (bool gTRUTH);

          /**
          *  Sets input string.
          *  @param pkcSTRING Source string.
          */
          void setInput (const char* pkcSTRING);

          /**
          *  Sets input string.
          *  @param rkySTRING Source string.
          */
          void setInput (const std::string& rkySTRING)
          {
            setInput (rkySTRING.c_str());
          }

          /**
          *  Sets input stream.
          *  @param rtISTREAM Source stream.
          */
          void setInput (std::basic_istream<char>& rtISTREAM);


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Matches the input string, using pattern \a pkcPATTERN_STRING. If there
          *  is  any argument in the pattern,  then it is written in  each argument
          *  after \a pkcPATTERN_STRING. The matched string is removed from stream.
          *
          *  \warning  You     must    supply    all    arguments    defined     in
          *            \a pkcPATTERN_STRING. If you don't want to supply arguments,
          *            then use a NULL value to tell  the function stopping getting
          *            arguments.
          *
          *  Declaration:  size_t scan (const char* pkcPATTERN_STRING[[, string&]*[, NULL]]) const;
          *
          *  @param  pkcPATTERN_STRING Pattern string.
          *  @return Length of string matched.
          */
          size_t scan (const char* pkcPATTERN_STRING...) const;

          /**
          *  Matches the string at \a rtSOURCE, with pattern \a pkcPATTERN_STRING.
          *  If there is any argument in the pattern, then it is forgotten. The
          *  matched string is not removed.
          *  @param  pkcPATTERN_STRING Pattern string.
          *  @return True if pattern \a pkcPATTERN_STRING matched input from stream.
          */
          bool match (const char* pkcPATTERN_STRING) const;


        protected:

          //
          //  C O N S T R U C T O R S
          //

          ///  Clears the local stream (if any).
          void clearStream (void);


        protected:

          //
          //  S E L E C T O R S
          //

          ///  Checks if the input stream (if any) is re-positionable.
          void checkStream (void) const;

          /**
          *  Instantiates pattern \a pkcPATTERN_STRING with all terminals defined,
          *  so  all the appearances of terminals defined are substituted by their
          *  definitions.
          *  @param pkcPATTERN_STRING Pattern string.
          *  @return Instantiated string.
          */
          TString instantiate (const char* pkcPATTERN_STRING) const;

          /**
          *  Matches the character \a cSOURCE, with the first in pattern
          *  \a pkcPATTERN_STRING.
          *  @param cSOURCE           Source character.
          *  @param pkcPATTERN_STRING Pattern string.
          *  @return True if first character in pattern \a pkcPATTERN_STRING
          *          matched character \a cSOURCE.
          */
          bool matchChars (char cSOURCE, const char* pkcPATTERN_STRING ) const;

      };  // class TMatcher

    }  // namespace regex

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_REGEX_MATCHER__
