/*
*  Name:      general-functions.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   General functions
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1999-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_GENERAL_FUNCTIONS__
#define _MPCL_UTIL_GENERAL_FUNCTIONS__

#include <cstdlib>
#include <functional>


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for UTIL.
  namespace util
  {

    //
    //  E X P O R T E D   F U N C T I O N S
    //

    /**
    *  Unary  function  object  to  append  an instance  (type TSourceItem)  to a
    *  pointer to object (type TTargetItem).
    */
    template <typename TTargetItem, typename TSourceItem>
    struct MAppend : public std::unary_function<TTargetItem, void>
    {

      public:

        ///  Source item for append from.
        const TSourceItem&   rktSourceItem;


      public:

        /**
        *  Builds a new instance for  appending \a rktSOURCE_ITEM to a provided
        *  \a TTargetItem instance (that must be a pointer).
        */
        MAppend (const TSourceItem& rktSOURCE_ITEM) :
          rktSourceItem (rktSOURCE_ITEM)            {}

        /**
        *  Appends  the item \a rktSourceItem to \a rtTARGET_ITEM (that must be
        *  have support for the operator '<<').
        *  @param rtTARGET_ITEM Item to append.
        */
        void operator () (TTargetItem& rtTARGET_ITEM) const
        {
          operator << (*rtTARGET_ITEM, rktSourceItem);
        }

    };  // struct MAppend

    ///  Unary function object to compare item features.
    template < typename TItem      ,
               typename TSizeType  ,
               typename MOperation ,
               typename MFunction  >
    struct MCompare : public std::unary_function<TItem, void>
    {

      public:

        ///  Function to apply to source objects.
        MFunction   mFunction;

        ///  Operation to apply to source objects.
        MOperation   mOperation;

        ///  Computed value.
        TSizeType   zComputedValue;


      public:

        ///  Builds a new instance.
        MCompare ( const TSizeType&  rkzVALUE     ,
                   const MOperation& rkmOPERATION ,
                   const MFunction&  rkmFUNCTION  ) :
          mFunction      (rkmFUNCTION)              ,
          mOperation     (rkmOPERATION)             ,
          zComputedValue (rkzVALUE)                 {}

        ///  Builds a new instance.
        MCompare (const TSizeType& rkzVALUE, const MFunction& rkmFUNCTION) :
          mFunction      (rkmFUNCTION)                                     ,
          mOperation     ()                                                ,
          zComputedValue (rkzVALUE)                                        {}

        /**
        *  Stores the result of the execution of the compare function onto
        *  \a rktITEM.
        *  @param rktITEM Item to delete.
        */
        void operator () (const TItem& rktITEM)
        {
          TSizeType   zItemResult (mFunction (rktITEM));

          if ( mOperation (zComputedValue, zItemResult) )
          {
            zComputedValue = zItemResult;
          }
        }

        /**
        *  Returns the computed value.
        *  @return The computed value.
        */
        operator TSizeType (void) const
        {
          return zComputedValue;
        }

    };  // struct MCompare

    ///  Unary function object to compare item features.
    template < typename TItem      ,
               typename TSizeType  ,
               typename MOperation ,
               typename MFunction  >
    struct MComparePointer : public std::unary_function<TItem, void>
    {

      public:

        ///  Function to apply to source objects.
        MFunction   mFunction;

        ///  Operation to apply to source objects.
        MOperation   mOperation;

        ///  Computed value.
        TSizeType   zComputedValue;


      public:

        ///  Builds a new instance.
        MComparePointer ( const TSizeType&  rkzVALUE     ,
                          const MOperation& rkmOPERATION ,
                          const MFunction&  rkmFUNCTION  ) :
          mFunction      (rkmFUNCTION)                     ,
          mOperation     (rkmOPERATION)                    ,
          zComputedValue (rkzVALUE)                        {}

        ///  Builds a new instance.
        MComparePointer (const TSizeType& rkzVALUE, const MFunction& rkmFUNCTION) :
          mFunction      (rkmFUNCTION)                                            ,
          mOperation     ()                                                       ,
          zComputedValue (rkzVALUE)                                               {}

        /**
        *  Stores the result of the execution of the compare function onto
        *  \a rktITEM.
        *  @param rktITEM Item to operate over.
        */
        void operator () (const TItem& rktITEM)
        {
          TSizeType   zItemResult (mFunction (*rktITEM));

          if ( mOperation (zComputedValue, zItemResult) )
          {
            zComputedValue = zItemResult;
          }
        }

        /**
        *  Stores the result of the execution of the compare function onto
        *  \a rtITEM.
        *  @param rtITEM Item to operate over.
        */
        void operator () (TItem& rtITEM)
        {
          TSizeType   zItemResult (mFunction (*rtITEM));

          if ( mOperation (zComputedValue, zItemResult) )
          {
            zComputedValue = zItemResult;
          }
        }

        /**
        *  Returns the computed value.
        *  @return The computed value.
        */
        operator TSizeType (void) const
        {
          return zComputedValue;
        }

    };  // struct MComparePointer

    ///  Unary function object to sum all the sizes of the items passed.
    template < typename TItem      ,
               typename TSizeType  ,
               typename MFunction  >
    struct MSum : public std::unary_function<TItem, void>
    {

      private:

        ///  Function.
        MFunction   mFunction;

        ///  Computed sum.
        TSizeType   zSum;


      public:

        /**
        *  Builds a new instance.
        *  @param rkmFUNCTION Function to execute over each instance.
        */
        MSum (const MFunction& rkmFUNCTION) :
          mFunction (rkmFUNCTION)           ,
          zSum      (0)                     {}

        /**
        *  Deletes the item \a rktITEM (that must be a pointer).
        *  @param rktITEM Item to delete.
        */
        void operator () (const TItem& rktITEM)
        {
          zSum += mFunction (rktITEM);
        }

        /**
        *  Returns the computed size.
        *  @return The computed size.
        */
        operator TSizeType (void) const
        {
          return zSum;
        }

    };  // struct MSum

    /**
    *  Unary  function  object to  sum  all the  sizes of  the  items passed  (as
    *  pointers).
    */
    template < typename TItem      ,
               typename TSizeType  ,
               typename MFunction  >
    struct MSumPointer : public std::unary_function<TItem, void>
    {

      private:

        ///  Function.
        MFunction   mFunction;

        ///  Computed sum.
        TSizeType   zSum;


      public:

        /**
        *  Builds a new instance.
        *  @param rkmFUNCTION Function to execute over each instance.
        */
        MSumPointer (const MFunction& rkmFUNCTION) :
          mFunction (rkmFUNCTION)                  ,
          zSum      (0)                            {}

        /**
        *  Deletes the item \a rktITEM (that must be a pointer).
        *  @param rktITEM Item to delete.
        */
        void operator () (const TItem& rktITEM)
        {
          zSum += mFunction (*rktITEM);
        }

        /**
        *  Returns the computed size.
        *  @return The computed size.
        */
        operator TSizeType (void) const
        {
          return zSum;
        }

    };  // struct MSumPointer

    ///  Unary function object to delete parameters.
    template <typename TItem>
    struct MDelete : public std::unary_function<TItem, void>
    {

        /**
        *  Deletes the item \a rktITEM (that must be a pointer).
        *  @param rktITEM Item to delete.
        */
        void operator () (const TItem& rktITEM) const
        {
          delete rktITEM;
        }

    };  // struct MDelete

    ///  Function object to generate random numbers.
    template <typename TItem>
    struct MRandomNumberGenerator
    {

        /**
        *  Generates  a random number between 0 to \a kzMAX (inclusive).  It uses
        *  this formula:
        *
        *    \f[f(n) = \frac {kzMAX + 1}{RAND\_MAX + 1} n\f]
        *
        *  Where:
        *
        *    \f[n \in [0, RAND\_MAX]\f]
        *
        *  It takes the higher-order bits of the random number which, usually are
        *  more random than the low-order  bits.  If you want more entropy in the
        *  generated random number, you must use \a std::srand().
        *  @param kzMAX Maximum value that it can return (inclusive).
        *  @return A random value from zero to kzMAX.
        *  @see std::rand(), std::srand().
        */
        TItem operator () (const size_t kzMAX) const
        {
          return TItem (((kzMAX + 1.0) / (RAND_MAX + 1.0)) * std::rand());
        }

    };  // struct MRandomNumberGenerator

  }  // namespace util

}  // namespace mpcl


#endif  // not _MPCL_UTIL_GENERAL_FUNCTIONS__
