/*
*  Name:         config_processor.hh
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Config processor
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_PREFS_CONFIG_PROCESSOR__
#define _MPCL_UTIL_PREFS_CONFIG_PROCESSOR__

#include <set>
#include <vector>
#include "../../io/streamable.hh"
#include "../../text/string.hh"
#include "../collection/string_to_string_map.hh"
#include "defs.hh"
#include "option.hh"


///  Name-space for MPCL.
namespace mpcl
{
  
  ///  Name-space for UTIL.
  namespace util
  {

    ///  Name-space for PREFS.
    namespace prefs
    {

      using io::IStreamable;
      using std::vector;
      using util::collection::TStringToStringMap;

      /**
      *  Configuration processor.  Parses  input  from command-line,  streams,  and
      *  configuration files.
      *
      *  The  configuration  file  is searched first in the current  directory  and
      *  then,   if  it isn't found, it is searched in the "home" directory  of the
      *  user (using environment variable 'HOME').
      *
      *  All the possible options must be  initially added to the instance with its
      *  name  in  the configuration file, its possible name in  command-line,  and
      *  its default value.
      *
      *  The  option  name  and  the option command-line name mustn't  contain  any
      *  punctuation character. The next regular expression, defines then valid set
      *  of characters:
      *
      *    [0-9a-zA-Z_.]+
      *
      *  This  class  makes distinctions between arguments (they don't  begin  with
      *  hyphen(s)),  and  options  (that  begin  with one  or  two  hyphens).  The
      *  arguments  can  be positioned in any position in  the  command-line  (even
      *  between options).
      *
      *  Option names  are kept  internally  in lowercase.  Lines in  configuration
      *  files beginning with '#' are ignored.
      */
      class TConfigProcessor : public IStreamable<>
      {

        public:

          typedef
            vector<TOption>::size_type
            size_type;

          ///  Vector of options type.
          typedef
            vector<TOption>
            TOptionVector;


        protected:

          /// Minimun padding size for Usage formatting
          static const unsigned int   _kuiMinPadding = 5;

          /// Column width for Usage formatting
          static const unsigned int   _kuiTerminalWidth = 80;

          ///  Boolean that tells if there is a config-file.
          bool   gInputConfigFile;

          ///  Boolean that tells if options have been processed.
          mutable bool   gOptionsProcessed;

          /**
          *  Integer with the number of command-line parameters. The term 'parameter'
          *  includes arguments and options.
          */
          int   iParametersCount;

          ///  Pointer to the array of command-line parameters.
          const char**   ppkcParametersList;

          /**
          *  Vector  with  command-line arguments.  The arguments  are  taken  from
          *  command-line and they are the parameters that aren't options.
          *
          *  For example:
          *
          *    executable_file --file=/etc/exec_conf main.c main.h
          *
          *  Where "main.c" and "main.h" are arguments.
          */
          mutable vector<TString>   tCommandLineArgumentsArray;

          ///  Map with environment variables.
          TStringToStringMap   tEnvironmentVariableMap;

          /**
          *  Vector with options.  Options  are  taken from command-line  and  from
          *  configuration file. From command-line they would appear as:
          *
          *    executable_file --file=/etc/exec_conf
          *    executable_file -f=/etc/exec_conf -f=/var/tmp
          *
          *  And from configuration file:
          *
          *    :
          *    file=/etc/exec_conf
          *    file=/var/tmp
          *    :
          *
          *  There can be quoted strings too:
          *
          *    :
          *    command="/usr/bin/ls -l"
          *    :
          */
          mutable TOptionVector   tOptionVector;

          ///  Set of valid name spaces.
          std::set<TString>   tValidNameSpaceSet;

          /**
          *  String with the name of the config-file (if any). If the file can't be
          *  opened,  then  it will be searched in the home directory  (environment
          *  variable 'HOME').
          */
          TString   yConfigFileName;

          /**
          *  String with current  name space.  It is empty when  options are in the
          *  root name space).
          */
          TString   yCurrentNameSpace;

          ///  String with the header of the usage message.
          TString   yUsageHeader;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TConfigProcessor (void)                 :
            IStreamable<>              ()         ,
            gInputConfigFile           (false)    ,
            gOptionsProcessed          (false)    ,
            iParametersCount           (0)        ,
            ppkcParametersList         (NULL)     ,        
            tCommandLineArgumentsArray ()         ,
            tEnvironmentVariableMap    ()         ,
            tOptionVector              ()         ,
            tValidNameSpaceSet         ()         ,
            yConfigFileName            ()         ,
            yCurrentNameSpace          ()         ,
            yUsageHeader               ("Usage:")
          {
            addEnvironmentVariable ("HOME");
          }

          /**
          *  Builds a new instance.
          *  @param iPARAMETER_COUNT Parameter count (in C language is \a argc).
          *  @param ppkcPARAMETERS   Parameter values (in C language is \a argv).
          *  @see argc, argv.
          */
          TConfigProcessor ( int          iPARAMETER_COUNT ,
                             const char** ppkcPARAMETERS   ) :
            IStreamable<>              ()                    ,
            gInputConfigFile           (false)               ,
            gOptionsProcessed          (false)               ,
            iParametersCount           (iPARAMETER_COUNT)    ,
            ppkcParametersList         (ppkcPARAMETERS)      ,
            tCommandLineArgumentsArray ()                    ,
            tEnvironmentVariableMap    ()                    ,
            tOptionVector              ()                    ,
            tValidNameSpaceSet         ()                    ,
            yConfigFileName            ()                    ,
            yCurrentNameSpace          ()                    ,
            yUsageHeader               ("Usage:")  
          {
            addEnvironmentVariable ("HOME");
            yConfigFileName = "conf." + programFileName();
          }

          ///  Destroyes the instance.
          virtual ~TConfigProcessor (void) {}

          /**
          *  Adds a new option.  Only options added with this member function,  are
          *  recognized later.
          *  @param rkyNAME     Name of the option.
          *  @param rkySHORTCUT Shortcut of the option.
          *  @pre ( !isThereOptionWithShortcut (rkySHORTCUT) )
          */
          virtual void addOption ( const TString& rkyNAME     ,
                                   const TString& rkySHORTCUT );

          /**
          *  Adds a new option.  Only options added with this member function,  are
          *  recognized later.
          *  @param rkyNAME          Name of the option.
          *  @param rkySHORTCUT      Shortcut of the option.
          *  @param rkyDEFAULT_VALUE Default value of the option.
          *  @pre ( !isThereOptionWithShortcut (rkySHORTCUT) )
          */
          virtual void addOption ( const TString& rkyNAME          ,
                                   const TString& rkySHORTCUT      ,
                                   const TString& rkyDEFAULT_VALUE );

          /**
          *  Adds a new accessible environment variable.  Only variables added with
          *  this member function, are recognized later.
          *
          *  If \a gREQUIRED is true and variable named \a pkcENVIRONMENT_NAME does
          *  not exist an exception is thrown,  else if \a gREQUIRED is false, then
          *  if variable doesn't exist, it is asigned an empty value.
          */
          void addEnvironmentVariable ( const char* pkcENVIRONMENT_NAME ,
                                        bool        gREQUIRED = false   )
            throw (TVariableNotFoundException);

          ///  Loads options from configuration file and command-line, if any.
          virtual void processOptions (void)
          {
            if ( !gOptionsProcessed )
            {
              processConfigFile();
              processCommandLine();
              gOptionsProcessed = true;
            }
          }

          ///  Sets configuration file name.
          void setConfigFileName (const char* pkcCONFIG_FILE_NAME)
          {
            gOptionsProcessed = false;
            yConfigFileName   = pkcCONFIG_FILE_NAME;
            gInputConfigFile  = true;
          }

          /**
          *  Sets command-line parameters.
          *  @param iPARAMETER_COUNT Parameter count (in C language is \a argc).
          *  @param ppkcPARAMETERS   Parameter values (in C language is \a argv).
          */
          void setCommandLineParameters ( int          iPARAMETER_COUNT ,
                                          const char** ppkcPARAMETERS   )
          {
            gOptionsProcessed  = false;
            iParametersCount   = iPARAMETER_COUNT;
            ppkcParametersList = ppkcPARAMETERS;
          }

          void setRootNameSpace (void)
          {
            yCurrentNameSpace.erase();
          }

          /**
          *  Sets current  name space.  Any option added after calling this method,
          *  will be renamed  prepending to its name,  the  string \a rkyNAME_SPACE
          *  plus a dot.
          *  @param rkyNAME_SPACE Name space.
          */
          void setCurrentNameSpace (const TString& rkyNAME_SPACE)
          {
            yCurrentNameSpace = rkyNAME_SPACE;
            yCurrentNameSpace.lowercase();
            if ( tValidNameSpaceSet.find (yCurrentNameSpace) == tValidNameSpaceSet.end() )
            {
              tValidNameSpaceSet.insert (yCurrentNameSpace);
            }
          }

          ///  Sets the usage header string
          void setUsageStringHeader (const TString& rkyUSAGE_HEADER)
          {
            yUsageHeader = rkyUSAGE_HEADER;
          }


        protected:

          //
          //  C O N S T R U C T O R S
          //

          ///  Updates options with command-line settings.
          virtual void processCommandLine (void);

          ///  Updates options with configuration file settings.
          virtual void processConfigFile (void);

          /**
          *  Reads the instance from stream \a rtSOURCE_ISTREAM.
          *  @param rtSOURCE_ISTREAM Source stream.
          */
          void read (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM);

          /**
          *  Updates option named \a rkyNAME with value \a rkyVALUE.
          *  @param rkyNAME  Name of the option.
          *  @param rkyVALUE Value to assigned.
          *  @return The option that has been updated.
          */
          virtual TOption& updateOptionWithName ( const TString& rkyNAME  ,
                                                  const TString& rkyVALUE );

          /**
          *  Updates option with shortcut \a rkySHORTCUT, with value \a rkyVALUE.
          *  @param rkySHORTCUT Shortcut of the option.
          *  @param rkyVALUE    Value to assigned.
          *  @return The option that has been updated.
          */
          virtual TOption& updateOptionWithShortcut ( const TString& rkySHORTCUT ,
                                                      const TString& rkyVALUE    );


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Gets argument value whose index is \a zARGUMENT_INDEX. Arguments don't
          *  include the program name.
          *  @param zARGUMENT_INDEX Argument index (from zero).
          *  @return The argument value.
          */
          TString argumentValue (size_type zARGUMENT_INDEX) const
          {
            return tCommandLineArgumentsArray [zARGUMENT_INDEX];
          }

          /**
          *  Gets environment variable value for the one named \a pkcENVIRONMENT_NAME
          *  (must be added before).
          */
          TString environmentValue (const char* pkcENVIRONMENT_NAME) const
          {
            return tEnvironmentVariableMap [pkcENVIRONMENT_NAME];
          }

          /**
          *  Returns true if has arguments in command line.
          *  @return True if numberOfArguments() is greater than 0.
          *  @see #numberOfArguments().
          */
          bool hasArguments (void) const
          {
            return ( numberOfArguments() > 0 );
          }

          ///  Gets number of arguments in command line.
          size_type numberOfArguments (void) const
          {
            return tCommandLineArgumentsArray.size();
          }

          /**
          *  Gets option corresponding to name \a pkcNAME.
          *  @param pkcNAME Option name.
          *  @return The corresponding option.
          */
          TOption& operator [] (const char* pkcNAME)
            throw (TVariableNotFoundException);

          /**
          *  Gets option corresponding to name \a pkcNAME.
          *  @param pkcNAME Option name.
          *  @return The corresponding option.
          */
          const TOption& operator [] (const char* pkcNAME) const
            throw (TVariableNotFoundException);

          const TOption& optionWithName (const TString& rkyNAME) const
          {
            return operator [] (rkyNAME.c_str());
          }

          ///  Gets option corresponding to name \a rkyNAME.
          TOption& optionWithName (const TString& rkyNAME)
          {
            return operator [] (rkyNAME.c_str());
          }

          ///  Print all options with it's help string, if any.
          TString formatUsageString (void) const;

          ///  Gets program name.
          const TString programFileName (void) const;

          ///  Saves the configuration file with the current values.
          void saveConfigFile (void) const
            throw (TFileErrorException);


        protected:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns true if there is an option named \a pkcNAME.
          *  @param pkcNAME Name of the option.
          *  @return True if there is an option named \a pkcNAME.
          */
          bool isThereOptionWithName (const char* pkcNAME) const;

          /**
          *  Returns true if there is an option with short-cut \a pkcSHORTCUT.
          *  @param pkcSHORTCUT Short-cut of the option.
          *  @return True if there is an option with short-cut \a pkcSHORTCUT.
          */
          bool isThereOptionWithShortcut (const char* pkcSHORTCUT) const;

          ///  Returns the longest string identifiying an option (needed for formatting)
          unsigned int longestOptionLength (void) const;

          /**
          *  Returns the substring from the  first character to the last occurrence
          *  of the dot character (but not including it). If the string has no name
          *  space, then it returns an empty string.
          *  @param rkyNAME Name.
          *  @return The name space part of the string.
          */
          TString nameSpace (const TString& rkyNAME) const;

          ///  Gets option name corresponding to shortcut \a rkySHORTCUT.
          TString nameForShortcut (const TString& rkySHORTCUT) const;

          ///  Gets option command-line name corresponding to name \a rkyNAME.
          TString shortcutForName (const TString& rkyNAME) const;

          /**
          *  Writes the instance onto stream \a rtTARGET_OSTREAM.
          *  @param rtTARGET_OSTREAM Target stream.
          *  @warning This method only writes valid name spaces.
          */
          void write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const;

      };  // class TConfigProcessor

    }  // namespace prefs

  }  // namespace util

}  // namespace mpcl


#endif  // not _MPCL_UTIL_PREFS_CONFIG_PROCESSOR__
