/*
*  Name:      variable_argument_array.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Variable argument list handler
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_VARIABLE_ARGUMENT_ARRAY__
#define _MPCL_UTIL_VARIABLE_ARGUMENT_ARRAY__

#include <cstdarg>
#include <vector>
#include "../exceptions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for UTIL.
  namespace util
  {

    /**
    *  Variable argument array.
    *
    *  Handles the problem of functions with variable number of arguments.
    *
    *  \warning  Whenever a \a TArgumentType instance, that has been passed
    *            as argument, is NULL, then the previous argument is the last
    *            argument in the list.
    */
    template <typename TArgumentType>
    class TVariableArgumentArray
    {

      private:

        /**
        *  Pointer to va_list type. It musts be initialized with
        *  std::va_start() function.
        *  @see std::va_list, std::va_start(), std::va_end().
        */
        va_list*   ptVa_list;

        /**
        *  True if the last call to function std::va_arg() failed to get a new
        *  argument. Whenever anyone makes a call to member function
        *  initialize(), this attribute is set to false value.
        *  @see #initialize().
        */
        mutable bool   gLastGetFailed;

        ///  Array with arguments.
        mutable std::vector<TArgumentType>   tArgumentArray;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TVariableArgumentArray (void) :
          ptVa_list      (NULL)       ,
          gLastGetFailed (false)      ,
          tArgumentArray ()           {}

        /**
        *  Sets ptVa_list with \a rktSOURCE_VA_LIST (must be initialized with
        *  std::va_start()).
        *  @param rtSOURCE_VA_LIST Source argument list.
        */
        void initialize (va_list& rtSOURCE_VA_LIST)
        {
          ptVa_list      = &rtSOURCE_VA_LIST;
          gLastGetFailed = false;
          tArgumentArray.clear();
        }


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Gets the argument at position \a zINDEX.
        *  @param zINDEX Argument index.
        *  @return The argument.
        */
        TArgumentType& operator [] (size_t zINDEX) const
          throw (TConstraintException)
        {
          if ( zINDEX >= size() )
          {
            throw TConstraintException ("out of limits", __FILE__, __LINE__);
          }
          return tArgumentArray [zINDEX];
        }

        /**
        *  Gets the arguments count.
        *  @return The arguments count.
        */
        size_t size (void) const;

    };  // class TVariableArgumentArray


    //
    //  C O N S T R U C T O R S
    //

    template <typename TArgumentType>
    inline size_t TVariableArgumentArray<TArgumentType>::size (void) const
    {

      if ( !gLastGetFailed )
      {
        int*   piTest = NULL;

        while ( !gLastGetFailed )
        {
          piTest = va_arg (*ptVa_list, int*);
          if ( !piTest )
          {
            gLastGetFailed = true;
            continue;
          }
          tArgumentArray.push_back ((TArgumentType) piTest);
        }
      }
      return tArgumentArray.size();

    }  // size()

  }  // namespace util

}  // namespace mpcl


#endif  // not _MPCL_UTIL_VARIABLE_ARGUMENT_ARRAY__
