/*
*  Name:      system.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Wrapper class for POSIX.1 compliant operating systems
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2001  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cerrno>
#include <cstdio>
#include <mpcl/system/posix_1/system.hh>
#include <sched.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


///  Name-space for MPCL.
namespace mpcl
{

  //
  //  L O C A L   V A R I A B L E S
  // 

  ///  POSIX.1 directory separator.
  static const char   _kcDirectorySeparator = '/';

  ///  POSIX.1 path separator.
  static const char   _kcPathSeparator = ':';

}  // namespace mpcl


//
//  S E L E C T O R S
//

void mpcl::system::TSystem::
sleep (std::size_t zSECONDS, std::size_t zNANOSECONDS) const
{
  struct timespec   tTimespec;

  tTimespec.tv_sec  = zSECONDS;
  tTimespec.tv_nsec = zNANOSECONDS;
  nanosleep (&tTimespec, NULL);
}


void mpcl::system::TSystem::
yield (void) const
{
  sched_yield();
}


int mpcl::system::TSystem::
getProcessId (void) const
{
  return getpid();
}


mpcl::text::TString mpcl::system::TSystem::
getCurrentDirectory (void) const
{

  using std::size_t;
  using text::TString;

  TString   yCurrentDirectory;
  char*     pcBuffer;
  size_t    zSize = 1024;
  
  pcBuffer = new char [zSize];
  while ( !getcwd (pcBuffer, zSize) )
  {
    if ( errno != ERANGE )
    {
      break;
    }
    else
    {
      zSize += 1024;
      delete[] pcBuffer;
      pcBuffer = new char [zSize];
    }
  }
  if ( pcBuffer )
  {
    yCurrentDirectory = pcBuffer;
    delete[] pcBuffer;
  }
  return yCurrentDirectory;

}  // getCurrentDirectory()


std::size_t mpcl::system::TSystem::
getFileSize (const char* pkcFILE_NAME) const
  throw (TErrorException)
{

  struct stat   sStat;
  std::size_t   zSize = 0;
  
  if ( stat (pkcFILE_NAME, &sStat) )
  {
    throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
  }
  else
  {
    zSize = sStat.st_size;
  }
  return zSize;
  
}  // getFileSize()


char mpcl::system::TSystem::
getDirectorySeparator (void) const
{
  return _kcDirectorySeparator;
}


char mpcl::system::TSystem::
getPathSeparator (void) const
{
  return _kcPathSeparator;
}


void mpcl::system::TSystem::
writeLogEntry ( const std::string& rkyMESSAGE ,
                ELogLevel          eLEVEL     ,
                ELogFacility       eFACILITY  ) const
{

  int   iPosixFacility = LOG_USER;
  int   iPosixLevel    = LOG_NOTICE;

  switch (eFACILITY)
  {
    case eAuthPriv: iPosixFacility = LOG_AUTHPRIV; break;
    case eCron:     iPosixFacility = LOG_CRON;     break;
    case eDaemon:   iPosixFacility = LOG_DAEMON;   break;
    case eKern:     iPosixFacility = LOG_KERN;     break;
    case eLocal0:   iPosixFacility = LOG_LOCAL0;   break;
    case eLocal1:   iPosixFacility = LOG_LOCAL1;   break;
    case eLocal2:   iPosixFacility = LOG_LOCAL2;   break;
    case eLocal3:   iPosixFacility = LOG_LOCAL3;   break;
    case eLocal4:   iPosixFacility = LOG_LOCAL4;   break;
    case eLocal5:   iPosixFacility = LOG_LOCAL5;   break;
    case eLocal6:   iPosixFacility = LOG_LOCAL6;   break;
    case eLocal7:   iPosixFacility = LOG_LOCAL7;   break;
    case eLpr:      iPosixFacility = LOG_LPR;      break;
    case eMail:     iPosixFacility = LOG_MAIL;     break;
    case eNews:     iPosixFacility = LOG_NEWS;     break;
    case eUser:     iPosixFacility = LOG_USER;     break;
    case eUucp:     iPosixFacility = LOG_UUCP;     break;
  }
  switch (eLEVEL)
  {
    case eEmerg:   iPosixLevel = LOG_EMERG;   break; 
    case eAlert:   iPosixLevel = LOG_ALERT;   break;
    case eCrit:    iPosixLevel = LOG_CRIT;    break;
    case eErr:     iPosixLevel = LOG_ERR;     break;
    case eWarning: iPosixLevel = LOG_WARNING; break;
    case eNotice:  iPosixLevel = LOG_NOTICE;  break;
    case eInfo:    iPosixLevel = LOG_INFO;    break;
    case eDebug:   iPosixLevel = LOG_DEBUG;   break;
  }
  syslog (iPosixLevel | iPosixFacility, rkyMESSAGE.c_str());

}  // writeLogEntry()


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    //
    //  E X P O R T E D   V A R I A B L E S
    //

    TSystem   tSystem;

  }  // namespace system

}  // namespace mpcl
