/*
*  Name:      IMutable.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Mutable interface
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for BEHAVIORAL design patterns.
package org.mpcl.designpatterns.behavioral;


/**
*  Mutable interface.
*  A class can implement  this interface when  it wants to  represent a mutable
*  object. A mutable object can have one or more mutators. A mutator may be any
*  object that implements interface \a IMutator. After a mutable instance needs
*  to   be  mutated   (changed),   an  application   calling   the   IMutable's
*  \a notifyMutators method,  causes all of its mutators to  be notified of the
*  need to be mutated by a call to their \a update method.  The order  in which
*  notifications will be delivered is unspecified.
*
*  Note that this notification mechanism is has nothing to do with  threads and
*  is completely separate from the \a wait and \a notify mechanism of the class
*  \a Object.
*/
public interface IMutable
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Adds an mutator to the  set of mutators for this object,  provided that it
  *  is not the same as some mutator already in the set.
  *  @param tMUTATOR The mutator to be added.
  */
  public void addMutator (IMutator tMUTATOR);

  /**
  *  Deletes an mutator from the set of mutators of this object.
  *  @param tMUTATOR The mutator to be removed.
  */
  public void deleteMutator (IMutator tMUTATOR);

  ///  Clears the mutator list so that this object no longer has any mutators.
  public void deleteMutators();

  /**
  *  Marks this mutable object as having been changed; the \a needMutation
  *  method will now return true.
  */
  public void setNeedMutation();


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the number of mutators of this mutable object.
  *  @return The number of mutators.
  */
  public int countMutators();

  /**
  *  Tests if this object need to be mutated.
  *  @return True if this object need to be mutated.
  */
  public boolean needMutation();

  /**
  *  If  this object  need  a mutation,  as  indicated  by  the \a needMutation
  *  method,   then  it   notifies   all  of   its  mutators   and   calls  the
  *  \a clearNeedMutation method  to indicate  that this  object has  no longer
  *  need for mutation.
  */
  public void notifyMutators();

  /**
  *  If  this object  need  a mutation,  as  indicated  by  the \a needMutation
  *  method,   then  it   notifies   all  of   its  mutators   and   calls  the
  *  \a clearNeedMutation method  to indicate  that this  object has  no longer
  *  need for mutation.
  *  @param tARGUMENT An argument passed to the mutator update method.
  *  @see IMutator
  */
  public void notifyMutators (Object tARGUMENT);

}  // interface IMutable
