/*
*  Name:      TFixedSizeDocumentFilter.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Filter for fixed size documents
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;


///  Filter for fixed size documents.
public class TFixedSizeDocumentFilter extends DocumentFilter
{

  ///  Maximum size of the text.
  private int   iMaxSize;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param iMAX_SIZE Maximum number of characters allowed.
  */
  public TFixedSizeDocumentFilter (int iMAX_SIZE)
  {
    iMaxSize = iMAX_SIZE;
  }

  /**
  *  Invoked prior to insertion of text into the specified Document. Subclasses
  *  that want to conditionally allow  insertion should  override this and only
  *  call  supers  implementation  as  necessary,  or call  directly  into  the
  *  \a FilterBypass.
  *  @param tFILTER_BYPASS The  \a FilterBypass  object  that  can be  used  to
  *                        mutate the \a Document object.
  *  @param iOFFSET        The offset  into the document to insert the content.
  *                        All  positions  that track  change at  or after  the
  *                        given location will move.
  *  @param yTEXT          The text to insert.
  *  @param tATTRIBUTE_SET The  attributes  to  associate   with  the  inserted
  *                        content.  This  may be  null  if there  are  no such
  *                        attributes.
  *  @pre ( iOFFSET >= 0 )
  *  @throws BadLocationException The  given  insert  position  is not a  valid
  *                               position within the document.
  */
  public void insertString ( DocumentFilter.FilterBypass tFILTER_BYPASS ,
                             int                         iOFFSET        ,
                             String                      yTEXT          ,
                             AttributeSet                tATTRIBUTE_SET )
    throws BadLocationException
  {
    replace (tFILTER_BYPASS, iOFFSET, 0, yTEXT, tATTRIBUTE_SET);
  }
    
  /**
  *  Invoked  prior to replacing a region of text in the  specified \a Document
  *  object.  Subclasses  that  want  to  conditionally  allow  replace  should
  *  override  this and only call  supers implementation as necessary,  or call
  *  directly into \a tFILTER_BYPASS object.
  *  @param tFILTER_BYPASS The  \a FilterBypass  object  that  can be  used  to
  *                        mutate the \a Document object.
  *  @param iOFFSET        The offset  into the document to insert the content.
  *                        All  positions  that track  change at  or after  the
  *                        given location will move.
  *  @param iLENGTH        Length of the text to delete.
  *  @param yTEXT          The text to insert,  \a null  indicates not  text to
  *                        insert.
  *  @param tATTRIBUTE_SET The  attributes  to  associate   with  the  inserted
  *                        content.  This  may be  null  if there  are  no such
  *                        attributes.
  *  @pre ( iOFFSET >= 0 )
  *  @throws BadLocationException The  given  insert  position  is not a  valid
  *                               position within the document.
  */
  public void replace ( DocumentFilter.FilterBypass tFILTER_BYPASS ,
                        int                         iOFFSET        ,
                        int                         iLENGTH        ,
                        String                      yTEXT          ,
                        AttributeSet                tATTRIBUTE_SET )
    throws BadLocationException
  {
    int   iNewLength;

    iNewLength = tFILTER_BYPASS.getDocument().getLength() - iLENGTH + yTEXT.length();
    if ( iNewLength <= iMaxSize)
    {
      tFILTER_BYPASS.replace (iOFFSET, iLENGTH, yTEXT, tATTRIBUTE_SET);
    }
    else
    {
      throw new BadLocationException ("new characters exceeds max size of document", iOFFSET);
    }
  }

}  // class TFixedSizeDocumentFilter
