/*
*  Name:      TModelViewComponentFacade.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Model view component facade
*  Date:      $Date: 2004-12-03 02:18:17 +0100 (vie, 03 dic 2004) $
*  Revision:  $Revision: 10528 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.JTextComponent;
import org.mpcl.util.ISimpleMap;


/**
*  Model view component facade.  This  class  makes that  the updates to a view
*  component  makes the model to be updated too.  This  component only  manages
*  \a JCheckBox,  \a JTextField,  \a JTextArea,  \a JLabel  and  TModelComboBox
*  components.
*  @see javax.swing.JCheckBox
*  @see javax.swing.JTextArea
*  @see javax.swing.JTextField
*  @see javax.swing.JLabel
*  @see org.mpcl.nui.TModelComboBox
*  @see org.mpcl.nui.TModelList
*/
public class TModelViewComponentFacade
  implements ActionListener        ,
             ChangeListener        ,
             DocumentListener      ,
             FocusListener         ,
             ItemListener          ,
             ListSelectionListener
{

  /**
  *  True if the component has changed since last call to the method
  *  \a write().
  */
  private boolean   gHasChanged;

  /**
  *  True if the component is  being read.  It is used to block \a write() from
  *  running where the component is being read.
  */
  private boolean   gIsReading;

  ///  Component.
  private JComponent   tComponent;

  ///  Component's key in \a tSimpleMap.
  private Object   tKey;

  ///  Simple map.
  private ISimpleMap   tSimpleMap;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tKEY        Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tCHECK_BOX  The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object     tKEY        ,
                                     JCheckBox  tCHECK_BOX  ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tCHECK_BOX;
    tKey        = tKEY;
    tSimpleMap  = tSIMPLE_MAP;
    tCHECK_BOX.addItemListener (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tCHECK_BOX  The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade (JCheckBox tCHECK_BOX, ISimpleMap tSIMPLE_MAP)
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tCHECK_BOX;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tCHECK_BOX.addItemListener (this);
  }

  /**
  *  Builds a new instance.
  *  @param tKEY        Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tSPINNER    The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object     tKEY        ,
                                     JSpinner   tSPINNER    ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    JSpinner.DefaultEditor   tSpinnerDefaultEditor;

    gHasChanged           = false;
    gIsReading            = false;
    tComponent            = tSPINNER;
    tKey                  = tKEY;
    tSimpleMap            = tSIMPLE_MAP;
    tSpinnerDefaultEditor = (JSpinner.DefaultEditor) tSPINNER.getEditor();
    tSpinnerDefaultEditor.getTextField().addFocusListener (this);
    tSPINNER.addChangeListener (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tSPINNER    The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade (JSpinner tSPINNER, ISimpleMap tSIMPLE_MAP)
  {
    JSpinner.DefaultEditor   tSpinnerDefaultEditor;

    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tSPINNER;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tSpinnerDefaultEditor = (JSpinner.DefaultEditor) tSPINNER.getEditor();
    tSpinnerDefaultEditor.getTextField().addFocusListener (this);
    tSPINNER.addChangeListener (this);
  }

  /**
  *  Builds a new instance.
  *  @param tKEY        Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tTEXT_AREA  The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object     tKEY        ,
                                     JTextArea  tTEXT_AREA  ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tTEXT_AREA;
    tKey        = tKEY;
    tSimpleMap  = tSIMPLE_MAP;
    tTEXT_AREA.addFocusListener (this);
    tTEXT_AREA.getDocument().addDocumentListener (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tTEXT_AREA  The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade (JTextArea tTEXT_AREA, ISimpleMap tSIMPLE_MAP)
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tTEXT_AREA;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tTEXT_AREA.addFocusListener (this);
    tTEXT_AREA.getDocument().addDocumentListener (this);
  }

  /**
  *  Builds a new instance.
  *  @param tKEY        Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tTEXT_FIELD The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object     tKEY        ,
                                     JTextField tTEXT_FIELD ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tTEXT_FIELD;
    tKey        = tKEY;
    tSimpleMap  = tSIMPLE_MAP;
    tTEXT_FIELD.addActionListener (this);
    tTEXT_FIELD.addFocusListener (this);
    tTEXT_FIELD.getDocument().addDocumentListener (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tTEXT_FIELD The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade (JTextField tTEXT_FIELD, ISimpleMap tSIMPLE_MAP)
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tTEXT_FIELD;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tTEXT_FIELD.addActionListener (this);
    tTEXT_FIELD.addFocusListener (this);
    tTEXT_FIELD.getDocument().addDocumentListener (this);
  }

  /**
  *  Builds a new instance.
  *  @param tKEY        Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tCOMPONENT  The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object     tKEY        ,
                                     JComponent tCOMPONENT  ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tCOMPONENT;
    tKey        = tKEY;
    tSimpleMap  = tSIMPLE_MAP;
    tCOMPONENT.addFocusListener (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tCOMPONENT  The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade (JComponent tCOMPONENT, ISimpleMap tSIMPLE_MAP)
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tCOMPONENT;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tCOMPONENT.addFocusListener (this);
  }

  /**
  *  Builds a new instance.
  *  @param tKEY             Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tMODEL_COMBO_BOX The view component.
  *  @param tSIMPLE_MAP      Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object         tKEY             ,
                                     TModelComboBox tMODEL_COMBO_BOX ,
                                     ISimpleMap     tSIMPLE_MAP      )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tMODEL_COMBO_BOX;
    tKey        = tKEY;
    tSimpleMap  = tSIMPLE_MAP;
    tMODEL_COMBO_BOX.addActionListener (this);
    tMODEL_COMBO_BOX.addItemListener   (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tMODEL_COMBO_BOX The view component.
  *  @param tSIMPLE_MAP      Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( TModelComboBox tMODEL_COMBO_BOX ,
                                     ISimpleMap     tSIMPLE_MAP      )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tMODEL_COMBO_BOX;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tMODEL_COMBO_BOX.addActionListener (this);
    tMODEL_COMBO_BOX.addItemListener   (this);
  }

  /**
  *  Builds a new instance.
  *  @param tKEY        Object identifier used with the map \a tSIMPLE_MAP.
  *  @param tMODEL_LIST The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( Object     tKEY        ,
                                     TModelList tMODEL_LIST ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tMODEL_LIST;
    tKey        = tKEY;
    tSimpleMap  = tSIMPLE_MAP;
    tMODEL_LIST.addListSelectionListener (this);
  }

  /**
  *  Builds a new instance. The key used is the own component.
  *  @param tMODEL_LIST The view component.
  *  @param tSIMPLE_MAP Map used for reading/writing component value.
  */
  public TModelViewComponentFacade ( TModelList tMODEL_LIST ,
                                     ISimpleMap tSIMPLE_MAP )
  {
    gHasChanged = false;
    gIsReading  = false;
    tComponent  = tMODEL_LIST;
    tKey        = this;
    tSimpleMap  = tSIMPLE_MAP;
    tMODEL_LIST.addListSelectionListener (this);
  }

  /**
  *  Invoked when an action occurs.
  *  @param tACTION_EVENT The \a ActionEvent object.
  *  @see java.awt.event.ActionListener#actionPerformed()
  */
  public void actionPerformed (ActionEvent tACTION_EVENT)
  {
    if ( hasChanged() )
    {
      write();
    }
  }

  /**
  *  Gives notification that an attribute or set of attributes changed.
  *  @param tDOCUMENT_EVENT The \a DocumentEvent object.
  *  @see javax.swing.event.DocumentListener#insertUpdate()
  */
  public void changedUpdate (DocumentEvent tDOCUMENT_EVENT)
  {
    setChanged (true);
  }

  /**
  *  Invoked when a component gains the keyboard focus.
  *  @param tFOCUS_EVENT The \a FocusEvent object.
  *  @see java.awt.event.FocusListener#focusGained()
  */
  public void focusGained (FocusEvent tFOCUS_EVENT) {}

  /**
  *  Invoked when a component loses the keyboard focus.
  *  @param tFOCUS_EVENT The \a FocusEvent object.
  *  @see java.awt.event.FocusListener#focusLost()
  */
  public void focusLost (FocusEvent tFOCUS_EVENT)
  {
    if ( hasChanged() )
    {
      write();
    }
  }

  /**
  *  Gives notification that there was  an insert into the document.  The range
  *  given by the \a DocumentEvent bounds the freshly inserted region.
  *  @param tDOCUMENT_EVENT The \a DocumentEvent object.
  *  @see javax.swing.event.DocumentListener#insertUpdate()
  */
  public void insertUpdate (DocumentEvent tDOCUMENT_EVENT)
  {
    setChanged (true);
  }

  /**
  *  Invoked when an item has been selected or deselected by the user. The code
  *  written for this method performs the operations that need to occur when an
  *  item is selected (or deselected).
  *  @param tITEM_EVENT The item event.
  *  @see java.awt.event.ItemListener#itemStateChanged()
  */
  public void itemStateChanged (ItemEvent tITEM_EVENT)
  {
    setChanged (true);
    if ( tITEM_EVENT.getSource() instanceof JCheckBox )
    {
      write();
    }
  }

  /**
  *  Updates the component text with the value taken from the \a tSimpleMap.
  *  @warning The value taken from \a tSimpleMap has to be a \a String object.
  */ 
  public void read()
  {

    setIsReading (true);
    if ( tComponent instanceof JCheckBox )
    {
      ((JCheckBox) tComponent).setSelected (((Boolean) getModelValue()).booleanValue());
    }
    else if ( tComponent instanceof JTextComponent )
    {
      ((JTextComponent) tComponent).setText ((String) getModelValue());
    }
    else if ( tComponent instanceof JLabel )
    {
      ((JLabel) tComponent).setText ((String) getModelValue());
    }
    else if ( tComponent instanceof JSpinner )
    {
      ((JSpinner) tComponent).setValue (getModelValue());
    }
    else if ( tComponent instanceof TModelComboBox )
    {
      ((TModelComboBox) tComponent).setSelectedCode ((Integer) getModelValue());
    }
    else if ( tComponent instanceof TModelList )
    {
      ((TModelList) tComponent).setSelectedCodes ((List) getModelValue());
    }
    setIsReading (false);
    setChanged (false);

  }  // read()

  /**
  *  Gives notification  that a portion of the document  has been removed.  The
  *  range is  given  in terms of  what  the  view  last saw  (that is,  before
  *  updating sticky positions).
  *  @param tDOCUMENT_EVENT The \a DocumentEvent object.
  *  @see javax.swing.event.DocumentListener#insertUpdate()
  */
  public void removeUpdate (DocumentEvent tDOCUMENT_EVENT)
  {
    setChanged (true);
  }

  /**
  *  Sets the has-changed property to \a gHAS_CHANGED.
  *  @param gHAS_CHANGED True if the value has changed.
  */
  public void setChanged (boolean gHAS_CHANGED)
  {
    gHasChanged = gHAS_CHANGED;
  }

  /**
  *  Sets the is-reading property to \a gIS_READING.
  *  @param gIS_READING True if there is an ongoing read of the value.
  *  @see #read()
  */
  public void setIsReading (boolean gIS_READING)
  {
    gIsReading = gIS_READING;
  }

  /**
  *  Invoked when the target of the listener has changed its state.
  *  @param tCHANGE_EVENT The \a ChangeEvent object.
  *  @see javax.swing.event.ChangeListener#stateChanged (ChangeEvent)
  */
  public void stateChanged (ChangeEvent tCHANGE_EVENT)
  {
    setChanged (true);
  }

  /**
  *  Called whenever the value of the selection changes.
  *  @param tLIST_SELECTION_EVENT The list selection event object.
  *  @see javax.swing.event.ListSelectionListener#valueChanged (ListSelectionEvent)
  */
  public void valueChanged (ListSelectionEvent tLIST_SELECTION_EVENT)
  {
    if ( !tLIST_SELECTION_EVENT.getValueIsAdjusting() )
    {
      setChanged (true);
      write();
    }
  }

  /**
  *  Updates the \a tSimpleMap with  the component value,  only if  there is no
  *  ongoing read.
  *  @see #isReading()
  */ 
  public void write()
  {

    Object   tValue = null;

    if ( !isReading() )
    {
      if ( tComponent instanceof JCheckBox )
      {
        tValue = new Boolean (((JCheckBox) tComponent).isSelected());
      }
      else if ( tComponent instanceof JTextComponent )
      {
        tValue = ((JTextComponent) tComponent).getText();
      }
      else if ( tComponent instanceof JLabel )
      {
        tValue = ((JLabel) tComponent).getText();
      }
      else if ( tComponent instanceof JSpinner )
      {
        tValue = ((JSpinner) tComponent).getValue();
      }
      else if ( tComponent instanceof TModelComboBox )
      {
        tValue = ((TModelComboBox) tComponent).getSelectedCode();
      }
      else if ( tComponent instanceof TModelList )
      {
        tValue = ((TModelList) tComponent).getSelectedCodes();
      }
      tSimpleMap.put (tKey, tValue);
      setChanged (false);
    }

  }  // write()


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the component.
  *  @return The component.
  */
  public JComponent getComponent()
  {
    return tComponent;
  }

  /**
  *  Returns the component value for its key from the model.
  *  @return The component value for its key from the model.
  */
  public Object getModelValue()
  {
    return tSimpleMap.get (tKey);
  }

  /**
  *  Returns true if the value of the has-changed property.
  *  @return True if the component has changed.
  *  @see #setChanged (boolean)
  */
  public boolean hasChanged()
  {
    return gHasChanged;
  }

  /**
  *  Returns true if the value of the is-reading property.
  *  @return True if the component is being read.
  *  @see #setIsReading (boolean)
  */
  public boolean isReading()
  {
    return gIsReading;
  }

}  // class TModelViewComponentFacade
