/*
*  Name:      TResourceSupplier.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Resource supplier class
*  Date:      $Date: 2004-05-13 15:16:05 +0200 (jue, 13 may 2004) $
*  Revision:  $Revision: 9385 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.awt.Color;
import java.util.MissingResourceException;
import java.util.ResourceBundle;


/**
*  Class for resources  containing  locale-specific objects.  This class uses a
*  resource bundle for managing the resources.  A unique resource name may have
*  different  objects bound;  it does the  trick by appending a suffix for each
*  kind of object it can have bound. 
*/
public class TResourceSupplier implements IResourceSupplier
{

  ///  Suffix for the resource bundle boolean key.
  private static String   _kyBooleanSuffix = ".boolean";

  ///  Suffix for the resource bundle color key.
  private static String   _kyColorSuffix = ".color";

  ///  Suffix for the resource bundle mnemonic key.
  private static String   _kyMnemonicSuffix = ".mnemonic";

  ///  Suffix for the resource bundle short text key.
  private static String   _kyShortTextSuffix = ".short_text";

  ///  Suffix for the resource bundle text key.
  private static String   _kyTextSuffix = ".text";

  ///  Suffix for the resource bundle tool tip text key.
  private static String   _kyToolTipTextSuffix = ".tooltip_text";

  ///  Resource bundle.
  private ResourceBundle   tResourceBundle;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tRESOURCE_BUNDLE Resource bundle.
  */
  public TResourceSupplier (ResourceBundle tRESOURCE_BUNDLE)
  {
    tResourceBundle = tRESOURCE_BUNDLE;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the resource string bound to the resource named \a kyRESOURCE_NAME
  *  and with the suffix \a kySUFFIX.
  *  @param kyRESOURCE_NAME Resource name.
  *  @param kySUFFIX        Resource suffix.
  *  @return The resource string.
  *  @throws NullPointerException     If key is null.
  *  @throws MissingResourceException If no object for the given key can be found.
  *  @throws ClassCastException       If the object found for the given key is not a string.
  */
  protected String getResourceString (final String kyRESOURCE_NAME, final String kySUFFIX)
  {
    return tResourceBundle.getString (kyRESOURCE_NAME + kySUFFIX);
  }
  

  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the boolean bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The boolean. If the resource boolean string is empty, it returns null.
  *  @warning The resource does not correspond to a resource in a ResourceBundle sense.
  */
  public Boolean getBoolean (final String kyRESOURCE_NAME)
  {
    String   yResourceString;

    yResourceString = getResourceString (kyRESOURCE_NAME, _kyBooleanSuffix);
    if ( yResourceString.length() == 0 )
    {
      yResourceString = null;
    }
    return Boolean.valueOf (yResourceString);
  }

  /**
  *  Returns the color bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The color. If the resource color string is empty, it returns null.
  *  @warning The resource does not correspond to a resource in a ResourceBundle sense.
  */
  public Color getColor (final String kyRESOURCE_NAME)
  {
    String   yResourceString;
    Color    tResourceColor = null;

    yResourceString = getResourceString (kyRESOURCE_NAME, _kyColorSuffix);
    if ( yResourceString.length() > 0 )
    {
      tResourceColor = Color.decode (yResourceString);
    }
    return tResourceColor;
  }

  /**
  *  Returns the mnemonic bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The mnemonic.
  *  @warning The resource does not correspond to a resource in a ResourceBundle sense.
  */
  public char getMnemonic (final String kyRESOURCE_NAME)
  {
    return getResourceString (kyRESOURCE_NAME, _kyMnemonicSuffix).charAt (0);
  }

  /**
  *  Returns the short text bound to the resource named \a kyRESOURCE_NAME.  If
  *  there not exists that short text, then the normal text is used.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The short text.
  *  @see #getText()
  */
  public String getShortText (final String kyRESOURCE_NAME)
  {
    return getResourceString (kyRESOURCE_NAME, _kyShortTextSuffix);
  }

  /**
  *  Returns the text bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The text.
  *  @warning The resource does not correspond to a resource in a ResourceBundle sense.
  */
  public String getText (final String kyRESOURCE_NAME)
  {
    return getResourceString (kyRESOURCE_NAME, _kyTextSuffix);
  }

  /**
  *  Returns the tool tip text bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The tool tip text.
  */
  public String getToolTipText (final String kyRESOURCE_NAME)
  {
    return getResourceString (kyRESOURCE_NAME, _kyToolTipTextSuffix);
  }

  /**
  *  Returns true if the resource named \a kyRESOURCE_NAME has bound a mnemonic.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return True if the resource has a mnemonic.
  */
  public boolean hasMnemonic (final String kyRESOURCE_NAME)
  {
    boolean   gHasMnemonic = true;

    try
    {
      getResourceString (kyRESOURCE_NAME, _kyMnemonicSuffix);
    }
    catch (final MissingResourceException ktEXCEPTION)
    {
      gHasMnemonic = false;
    }
    return gHasMnemonic;
  }

  /**
  *  Returns true if the resource named \a kyRESOURCE_NAME has bound a tool tip
  *  text.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return True if the resource has a tool tip text.
  */
  public boolean hasToolTipText (final String kyRESOURCE_NAME)
  {
    boolean   gHasToolTipText = true;

    try
    {
      getResourceString (kyRESOURCE_NAME, _kyToolTipTextSuffix);
    }
    catch (final MissingResourceException ktEXCEPTION)
    {
      gHasToolTipText = false;
    }
    return gHasToolTipText;
  }

}  // class TResourceSupplier
