/*
*  Name:      TTable.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   A table with better user control for its appearance.
*  Date:      $Date: 2004-08-09 19:12:59 +0200 (lun, 09 ago 2004) $
*  Revision:  $Revision: 9876 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableColumn;
import org.mpcl.nui.table.ITableAppearanceModel;


/**
*  Table component. It enhances \a JTable by adding a table appearance model that
*  lets the user select the sizes for columns.
*/
public class TTable extends JTable
{

  ///  Table appearance model.
  private ITableAppearanceModel   tTableAppearanceModel = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Sets the minimum widths for some kind of columns. It also sets the preferred
  *  dimension for the scrollable viewport.
  */
  private void setMinWidths()
  {
    Enumeration   I;
    int           iNewMinWidth;
    TableColumn   tTableColumn;

    if ( tTableAppearanceModel != null )
    {
      I = getColumnModel().getColumns();
      while ( I.hasMoreElements() )
      {
        tTableColumn = (TableColumn) I.nextElement();
        iNewMinWidth = tTableAppearanceModel.getColumnMinWidth (tTableColumn.getModelIndex());
        tTableColumn.setMinWidth (iNewMinWidth);
      }
    }
  }


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Causes this table to lay out its rows and columns.  It also recomputes the
  *  minimum widths of columns using the table appearance model.
  *  @see javax.swing.JTable#doLayout()
  */
  public void doLayout()
  {
    setMinWidths();
    setPreferredScrollableViewportSize();
    super.doLayout();
  }

  /**
  *  Sets the appearance model.
  *  @param tTABLE_APPEARANCE_MODEL The table appearance model.
  */
  public void setAppearanceModel (ITableAppearanceModel tTABLE_APPEARANCE_MODEL)
  {
    tTableAppearanceModel = tTABLE_APPEARANCE_MODEL;
  }

  ///  Sets the preferred dimension for the scrollable viewport.
  public void setPreferredScrollableViewportSize()
  {
    Dimension   tNewDimension;

    if ( tTableAppearanceModel != null )
    {
      tNewDimension = tTableAppearanceModel.getPreferredScrollableViewportSize();
      setPreferredScrollableViewportSize (tNewDimension);
    }
  }

  /**
  *  Invoked when this table's \a TableModel generates a \ a TableModelEvent.
  *  @param tTABLE_MODEL_EVENT The table model event.
  */
  public void tableChanged (TableModelEvent tTABLE_MODEL_EVENT)
  {
    Dimension   tNewDimension;

    switch (tTABLE_MODEL_EVENT.getType())
    {
      case TableModelEvent.INSERT:
      case TableModelEvent.DELETE:
      {
        setPreferredScrollableViewportSize();
        break;
      }
    }
    super.tableChanged (tTABLE_MODEL_EVENT);
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Gets the appearance model.
  *  @return The table appearance model.
  */
  public ITableAppearanceModel getAppearanceModel()
  {
    return tTableAppearanceModel;
  }

  /**
  *  Returns the preferred size of the viewport for a view component.
  *  @return The preferredSize of a \a JViewport whose view is this \a Scrollable.
  *  @see javax.swing.Scrollable#getPreferredScrollableViewportSize()
  *  @see javax.swing.JTable#getPreferredScrollableViewportSize()
  */
  public Dimension getPreferredScrollableViewportSize()
  {
    Dimension   tDimension;

    if ( tTableAppearanceModel == null )
    {
      tDimension = super.getPreferredScrollableViewportSize();
    }
    else
    {
      tDimension = tTableAppearanceModel.getPreferredScrollableViewportSize();
    }
    return tDimension;
  }

  /**
  *  It uses the appearance model to get the tool-tip for a table cell.
  *  @param tMOUSE_EVENT The mouse event.
  *  @return The tool-tip text.
  */
  public String getToolTipText (MouseEvent tMOUSE_EVENT)
  {

    int      iColumnIndex;
    int      iRealColumnIndex;
    int      iRowIndex;
    Point    tMousePoint;
    String   yToolTipText;

    if ( tTableAppearanceModel == null )
    {
      yToolTipText = super.getToolTipText (tMOUSE_EVENT);
    }
    else
    {
      tMousePoint      = tMOUSE_EVENT.getPoint();
      iRowIndex        = rowAtPoint (tMousePoint);
      iColumnIndex     = columnAtPoint (tMousePoint);
      iRealColumnIndex = convertColumnIndexToModel (iColumnIndex);
      yToolTipText     = tTableAppearanceModel.getToolTipText (iRowIndex, iRealColumnIndex);
    }
    return yToolTipText;

  }  // getToolTipText()

}  // class TTable
