/*
*  Name:        TTreeTable.java
*  Author:      Philip Milne
*               Scott Violet
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     A table that can hold a tree column.
*  Date:        $Date: 2004-10-11 00:59:30 +0200 (lun, 11 oct 2004) $
*  Revision:    $Revision: 10229 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.awt.Dimension;
import javax.swing.LookAndFeel;
import javax.swing.tree.TreePath;
import org.mpcl.nui.treetable.ITreeTableAppearanceModel;
import org.mpcl.nui.treetable.ITreeTableCellRenderer;
import org.mpcl.nui.treetable.ITreeTableModel;
import org.mpcl.nui.treetable.TTree;
import org.mpcl.nui.treetable.TTreeSelectionModel;
import org.mpcl.nui.treetable.TTreeTableAppearanceModelToTableAppearanceModelAdapter;
import org.mpcl.nui.treetable.TTreeTableCellEditor;
import org.mpcl.nui.treetable.TTreeTableCellRenderer;
import org.mpcl.nui.treetable.TTreeTableModelToTableModelAdapter;


/**
*  Tree-table component.  It  uses a \a JTree as a renderer  and editor for the
*  cells in a particular column in the \a TTable.
*/
public class TTreeTable extends TTable
{

  ///  Tree. It will be used as renderer and editor.
  protected TTree   tTree;

  /**
  *  Tree-table model. It is a table model representing the visible rows in the
  *  tree.
  */
  protected ITreeTableModel   tTreeTableModel;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tTREE_TABLE_MODEL The tree-table model.
  */
  public TTreeTable (ITreeTableModel tTREE_TABLE_MODEL)
  {

    super();

    TTreeSelectionModel   tTreeSelectionModel;

    tTreeTableModel = tTREE_TABLE_MODEL;
    tTree           = new TTree (this, tTreeTableModel);
    super.setModel (new TTreeTableModelToTableModelAdapter (tTreeTableModel, tTree));

    //
    //  Forces the JTable and JTree to share their row selection models.
    //
    tTreeSelectionModel = new TTreeSelectionModel (tTree);
    tTree.setSelectionModel (tTreeSelectionModel);
    setSelectionModel (tTreeSelectionModel.getListSelectionModel()); 

    //
    //  Installs the tree cell renderer and editor.
    //
    setDefaultRenderer (new TTreeTableCellRenderer (this));
    setDefaultEditor (ITreeTableModel.class, new TTreeTableCellEditor (this));

    //
    //  Shows no grid.
    //
    setShowGrid (false);

    //
    //  Sets no intercell spacing.
    setIntercellSpacing (new Dimension (0, 0));	

  }  // TTreeTable()

  ///  Collapses all nodes in the tree.
  public void collapseTreeAll()
  {
    tTree.collapseAll (new TreePath (tTree.getModel().getRoot()));
  }

  ///  Expands all nodes in the tree.
  public void expandTreeAll()
  {
    tTree.expandAll (new TreePath (tTree.getModel().getRoot()));
  }

  /**
  *  Sets a default cell  renderer to be used if no renderer  has been set in a
  *  \a TableColumn.  If \a tTREE_TABLE_CELL_RENDERER is null,  then it removes
  *  the default renderer for the column class \a ITreeTableModel.
  *  @param tTREE_TABLE_CELL_RENDERER
  *         Default cell renderer to be used for the column class \a ITreeTableModel.
  */
  public void setDefaultRenderer (ITreeTableCellRenderer tTREE_TABLE_CELL_RENDERER)
  {

    ITreeTableCellRenderer   tTreeTableCellRenderer;

    if ( tTREE_TABLE_CELL_RENDERER != null )
    {
      tTree.setCellRenderer (tTREE_TABLE_CELL_RENDERER);
      setDefaultRenderer (ITreeTableModel.class, tTREE_TABLE_CELL_RENDERER);
    }
    else
    {
      tTreeTableCellRenderer = new TTreeTableCellRenderer (this);
      tTree.setCellRenderer (tTreeTableCellRenderer);
      setDefaultRenderer (ITreeTableModel.class, tTreeTableCellRenderer);
    }

  }  // setDefaultRenderer()

  /**
  *  Sets the last visible row for the tree renderer.
  *  @param iLAST_VISIBLE_ROW The tree last visible row.
  */
  public void setLastVisibleRow (int iLAST_VISIBLE_ROW)
  {
    tTree.setLastVisibleRow (iLAST_VISIBLE_ROW);
  }

  /**
  *  Overridden to also pass the new row height to the tree.
  *  @param iROW_HEIGHT The new row height.
  *  @see javax.swing.JTable#setRowHeight()
  */
  public void setRowHeight (int iROW_HEIGHT)
  { 
    super.setRowHeight (iROW_HEIGHT); 
    if ( ( tTree != null ) && ( tTree.getRowHeight() != iROW_HEIGHT ) )
    {
      tTree.setRowHeight (getRowHeight());
    }
  }

  /**
  *  Sets the appearance model.
  *  @param tTREE_TABLE_APPEARANCE_MODEL The table appearance model.
  */
  public void setAppearanceModel (ITreeTableAppearanceModel tTREE_TABLE_APPEARANCE_MODEL)
  {
    setAppearanceModel (new TTreeTableAppearanceModelToTableAppearanceModelAdapter (tTREE_TABLE_APPEARANCE_MODEL, tTree));
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Gets the default cell renderer.
  *  @return The default cell renderer to be used for the column class \a ITreeTableModel.
  */
  public ITreeTableCellRenderer getDefaultRenderer()
  {
    return (ITreeTableCellRenderer) tTree.getCellRenderer();
  }

  /*
  *  Workaround for BasicTableUI anomaly. Make sure the UI never tries to paint
  *  the editor. The UI currently uses different techniques to paint the
  *  renderers and editors and overriding \a setBounds() below is not the right
  *  thing to do for an editor.  Returning -1 for the editing row in this case,
  *  ensures the editor is never painted.
  *  @return The editing row.
  *  @see javax.swing.JTable#getEditingRow()
  */
  public int getEditingRow()
  {
    return ( getColumnClass (editingColumn) == ITreeTableModel.class ) ? -1 : editingRow;
  }

  /**
  *  Returns the node at row \a iROW.
  *  @return The node.
  */
  public Object getNodeForRow (int iROW)
  {
    return tTree.getPathForRow (iROW).getLastPathComponent();         
  }

  /**
  *  Returns the tree that is being shared between the model.
  *  @return The tree.
  */
  public TTree getTree()
  {
    return tTree;
  }

  /**
  *  Returns the tree-table model.
  *  @return The tree-table model.
  */
  public ITreeTableModel getTreeTableModel()
  {
    return tTreeTableModel;
  }

  /**
  *  Overridden to message super and forward the method to the tree.  Since the
  *  tree is not  actually in  the component  hierarchy,  it will never receive
  *  this unless it forwards it in this manner.
  */
  public void updateUI()
  {

    super.updateUI();
    if ( tTree != null )
    {
      tTree.updateUI();

      //
      //  It does this so  that the editor is  referencing the current renderer
      //  from the tree.  The renderer  can potentially  change each  time  LAF
      //  changes.
      //
      setDefaultEditor (ITreeTableModel.class, new TTreeTableCellEditor (this));
    }

    //
    //  Use the tree's default foreground and background colors in the table.
    //
    LookAndFeel.installColorsAndFont (this, "Tree.background", "Tree.foreground", "Tree.font");

  }  // updateUI()

}  // class TTreeTable
