/*
*  Name:      TThinBevelBorder.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Class that implements a simple 1 line bevel border
*  Date:      $Date: 2004-01-02 12:43:42 +0100 (vie, 02 ene 2004) $
*  Revision:  $Revision: 8665 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for BORDER (border support classes).
package org.mpcl.nui.border;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import javax.swing.border.Border;


///  Class for bevel borders with one single line.
public class TThinBevelBorder implements Border
{

  ///  Bevel type.
  private int   iBevelType;

  ///  Highlight color.
  private Color   tHighlightColor;

  ///  Shadow color.
  private Color   tShadowColor;

  ///  Class for bevel types.
  public static class _TBevelTypes
  {
    ///  Lowered bevel border.
    public static final int   _kiLowered = 0;

    ///  Lowered bevel border without the right border.
    public static final int   _kiLoweredWithoutRightBorder = 1;

    ///  Raised bevel border.
    public static final int   _kiRaised = 2;

    ///  Raised bevel border without the right border.
    public static final int   _kiRaisedWithoutRightBorder = 3;
  }


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance. The new bevel border will have the specified type
  *  and  the  colors  will  be derived  from  the background  color  of  the
  *  component passed into the \a paintBorder() method.
  *  @param iBEVEL_TYPE The type of bevel for the border
  *  @see #paintBorder()
  */
  public TThinBevelBorder (int iBEVEL_TYPE)
  {
    iBevelType      = iBEVEL_TYPE;
    tHighlightColor = null;
    tShadowColor    = null;
  }

  /**
  *  Builds a new instance.
  *  @param iBEVEL_TYPE      The type of bevel for the border.
  *  @param tHIGHLIGHT_COLOR The color to use for the bevel highlight.
  *  @param tSHADOW_COLOR    The color to use for the bevel shadow.
  */
  public TThinBevelBorder (int iBEVEL_TYPE, Color tHIGHLIGHT_COLOR, Color tSHADOW_COLOR)
  {
    iBevelType      = iBEVEL_TYPE;
    tHighlightColor = tHIGHLIGHT_COLOR;
    tShadowColor    = tSHADOW_COLOR;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the insets of the border.
  *  @param tCOMPONENT The component for which this border insets value applies.
  *  @return The insets.
  */
  public Insets getBorderInsets (Component tCOMPONENT)
  {
    return new Insets (1, 1, 1, 1);
  }

  /**
  *  Returns the  highlight color  of the  bevel  border  when rendered  on the
  *  specified component. If no highlight color was specified at instantiation,
  *  the highlight color is derived  from the specified  component's background
  *  color.
  *  @param tCOMPONENT The component for which the highlight may be derived.
  *  @return The highlight color.
  */
  public Color getHighlightColor (Component tCOMPONENT)
  {
    return ( tHighlightColor != null )? tHighlightColor : tCOMPONENT.getBackground().brighter();
  }

  /**
  *  Returns  the  shadow color  of  the  bevel border  when  rendered  on  the
  *  specified component.  If no  shadow color  was specified at instantiation,
  *  the shadow color  is derived  from the  specified  component's  background
  *  color.
  *  @param tCOMPONENT The component for which the shadow may be derived.
  *  @return The shadow color.
  */
  public Color getShadowColor (Component tCOMPONENT)
  {
    return ( tShadowColor != null )? tShadowColor : tCOMPONENT.getBackground().darker();
  }

  /**
  *  Returns whether or not the border is opaque.
  *  @return True if the border is opaque.
  */
  public boolean isBorderOpaque()
  {
    return true;
  }

  /**
  *  Paints the border for the specified component with the specified position
  *  and size.
  *  @param tCOMPONENT The component for which this border is being painted.
  *  @param tGRAPHICS  The paint graphics.
  *  @param iX         The x position of the painted border.
  *  @param iY         The y position of the painted border.
  *  @param iWIDTH     The width of the painted border.
  *  @param iHEIGHT    The height of the painted border.
  *  @see javax.swing.border.Border#paintBorder()
  */
  public void paintBorder ( Component tCOMPONENT ,
                            Graphics  tGRAPHICS  ,
                            int       iX         ,
                            int       iY         ,
                            int       iWIDTH     ,
                            int       iHEIGHT    )
  {

    switch (iBevelType)
    {
      case _TBevelTypes._kiLowered:
      {
        paintLoweredBevel (tCOMPONENT, tGRAPHICS, iX, iY, iWIDTH, iHEIGHT, true);
        break;
      }
      case _TBevelTypes._kiLoweredWithoutRightBorder:
      {
        paintLoweredBevel (tCOMPONENT, tGRAPHICS, iX, iY, iWIDTH, iHEIGHT, false);
        break;
      }
      case _TBevelTypes._kiRaised:
      {
        paintRaisedBevel (tCOMPONENT, tGRAPHICS, iX, iY, iWIDTH, iHEIGHT, true);
        break;
      }
      case _TBevelTypes._kiRaisedWithoutRightBorder:
      {
        paintRaisedBevel (tCOMPONENT, tGRAPHICS, iX, iY, iWIDTH, iHEIGHT, false);
        break;
      }
    }

  }  // paintBorder()

  /**
  *  Paints the border for the specified component with the specified position
  *  and size.
  *  @param tCOMPONENT         The component for which this border is being painted.
  *  @param tGRAPHICS          The paint graphics.
  *  @param iX                 The x position of the painted border.
  *  @param iY                 The y position of the painted border.
  *  @param iWIDTH             The width of the painted border.
  *  @param iHEIGHT            The height of the painted border.
  *  @param gDRAW_RIGHT_BORDER True if it must paint the right border.
  *  @see javax.swing.border.Border#paintBorder()
  */
  protected void paintLoweredBevel ( Component tCOMPONENT         ,
                                     Graphics  tGRAPHICS          ,
                                     int       iX                 ,
                                     int       iY                 ,
                                     int       iWIDTH             ,
                                     int       iHEIGHT            ,
                                     boolean   gDRAW_RIGHT_BORDER )
  {
    Color   tOldColor = tGRAPHICS.getColor();

    tGRAPHICS.translate (iX, iY);
    tGRAPHICS.setColor (getShadowColor (tCOMPONENT));
    tGRAPHICS.drawLine (0, 0, 0,          iHEIGHT - 1);
    tGRAPHICS.drawLine (1, 0, iWIDTH - 1, 0);

    tGRAPHICS.setColor (getHighlightColor (tCOMPONENT));
    tGRAPHICS.drawLine (1, iHEIGHT - 1, iWIDTH - 1, iHEIGHT - 1);
    if ( gDRAW_RIGHT_BORDER )
    {
      tGRAPHICS.drawLine (iWIDTH - 1, 1, iWIDTH - 1, iHEIGHT - 2);
    }

    //
    //  Back-up changes in graphics object.
    //
    tGRAPHICS.translate (-iX, -iY);
    tGRAPHICS.setColor (tOldColor);
  }

  /**
  *  Paints the border for the specified component with the specified position
  *  and size.
  *  @param tCOMPONENT         The component for which this border is being painted.
  *  @param tGRAPHICS          The paint graphics.
  *  @param iX                 The x position of the painted border.
  *  @param iY                 The y position of the painted border.
  *  @param iWIDTH             The width of the painted border.
  *  @param iHEIGHT            The height of the painted border.
  *  @param gDRAW_RIGHT_BORDER True if it must paint the right border.
  *  @see javax.swing.border.Border#paintBorder()
  */
  protected void paintRaisedBevel ( Component tCOMPONENT         ,
                                    Graphics  tGRAPHICS          ,
                                    int       iX                 ,
                                    int       iY                 ,
                                    int       iWIDTH             ,
                                    int       iHEIGHT            ,
                                    boolean   gDRAW_RIGHT_BORDER )
  {
    Color   tOldColor = tGRAPHICS.getColor();

    tGRAPHICS.translate (iX, iY);
    tGRAPHICS.setColor (getHighlightColor (tCOMPONENT));
    tGRAPHICS.drawLine (0, 0, 0,          iHEIGHT - 1);
    tGRAPHICS.drawLine (1, 0, iWIDTH - 1, 0);

    tGRAPHICS.setColor (getShadowColor (tCOMPONENT));
    tGRAPHICS.drawLine (0, iHEIGHT - 1, iWIDTH, iHEIGHT - 1);
    if ( gDRAW_RIGHT_BORDER )
    {
      tGRAPHICS.drawLine (iWIDTH - 1, 0, iWIDTH - 1, iHEIGHT - 1);
    }

    //
    //  Back-up changes in graphics object.
    //
    tGRAPHICS.translate (-iX, -iY);
    tGRAPHICS.setColor (tOldColor);
  }

}  // class TThinBevelBorder
