/*
*  Name:      TAbstractDialog.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Abstract dialog class
*  Date:      $Date: 2004-12-07 02:48:27 +0100 (mar, 07 dic 2004) $
*  Revision:  $Revision: 10538 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI DIALOG.
package org.mpcl.nui.dialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.Border;
import org.mpcl.designpatterns.mvc.controller.TAbstractController;
import org.mpcl.nui.TAbstractInputVerifier;


///  Base dialog class.
public abstract class TAbstractDialog extends JDialog
{

  /**
  *  Registered components to be verified in method \a isValid().
  *  @see #isValid(), java.util.List.
  */
  private List   tVerifiedComponentList;

  ///  Basic empty border space length.
  protected static final int   _kiEmptyBorderLength = 10;

  /**
  *  Controller instance.
  *  @see org.mpcl.designpatterns.mvc.TAbstractController.
  */
  protected TAbstractController   tController;

  /**
  *  Main component container panel. It uses a border layout.
  *  @see java.awt.BorderLayout.
  */
  protected JPanel   tMainPanel;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.  Creates a non-modal  dialog  without  a title  and
  *  without a specified \a Frame owner.  A shared, hidden frame will be set as
  *  the owner of the dialog.
  *  @param tCONTROLLER Controller instance.
  */
  public TAbstractDialog (TAbstractController tCONTROLLER)
  {
    super();
    tController            = tCONTROLLER;
    tMainPanel             = new JPanel();
    tVerifiedComponentList = new ArrayList();
    tMainPanel.setLayout (new BorderLayout());
    tMainPanel.setBorder (BorderFactory.createEmptyBorder (_kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength));
    getContentPane().setLayout (new GridLayout (1, 1));
    getContentPane().add (tMainPanel);
  }

  /**
  *  Builds a new instance.
  *  @param tDIALOG_OWNER The \a Dialog object owner of this dialog.
  *  @param yTITLE        Title of the dialog.
  *  @param tCONTROLLER   Controller instance.
  *  @param gIS_MODAL     True if the dialog is modal (it does not permit other
  *                       interaction with the application).
  */
  public TAbstractDialog ( Dialog              tDIALOG_OWNER ,
                           String              yTITLE        ,
                           TAbstractController tCONTROLLER   ,
                           boolean             gIS_MODAL     )
  {
    super (tDIALOG_OWNER, yTITLE, gIS_MODAL);
    tController            = tCONTROLLER;
    tMainPanel             = new JPanel();
    tVerifiedComponentList = new ArrayList();
    tMainPanel.setLayout (new BorderLayout());
    tMainPanel.setBorder (BorderFactory.createEmptyBorder (_kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength));
    getContentPane().setLayout (new GridLayout (1, 1));
    getContentPane().add (tMainPanel);
  }

  /**
  *  Builds a new instance.
  *  @param tFRAME_OWNER The \a Frame object owner of this dialog.
  *  @param yTITLE       Title of the dialog.
  *  @param tCONTROLLER  Controller instance.
  *  @param gIS_MODAL    True if the dialog is modal  (it does not permit other
  *                      interaction with the application).
  */
  public TAbstractDialog ( Frame               tFRAME_OWNER ,
                           String              yTITLE       ,
                           TAbstractController tCONTROLLER  ,
                           boolean             gIS_MODAL    )
  {
    super (tFRAME_OWNER, yTITLE, gIS_MODAL);
    tController            = tCONTROLLER;
    tMainPanel             = new JPanel();
    tVerifiedComponentList = new ArrayList();
    tMainPanel.setLayout (new BorderLayout());
    tMainPanel.setBorder (BorderFactory.createEmptyBorder (_kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength));
    getContentPane().setLayout (new GridLayout (1, 1));
    getContentPane().add (tMainPanel);
  }

  /**
  *  Adds a component to the container of components to be verified.
  *  @param tCOMPONENT Component to be verified.
  */
  public void addToVerifieds (JComponent tCOMPONENT)
  {
    tVerifiedComponentList.add (tCOMPONENT);
  }

  ///  Initializes dialog.
  public void initialize() {}

  /**
  *  Adds the dialog to the observed objects.
  *  @see java.util.Observer.
  */
  void attachToObservable() {}

  /**
  *  Removes the dialog from the observed objects.
  *  @see java.util.Observer.
  */
  void detachFromObservable() {}


  //
  //  S E L E C T O R S
  //

  /**
  *  Creates a compound border.
  *  @return A compound border.
  */
  public Border createBorder()
  {
    Border   tInsideBorder;
    Border   tOutsideBorder;

    tOutsideBorder = BorderFactory.createLineBorder (Color.gray);
    tInsideBorder  = BorderFactory.createEmptyBorder (_kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength, _kiEmptyBorderLength);
    return BorderFactory.createCompoundBorder (tOutsideBorder, tInsideBorder);
  }

  /**
  *  Returns the main panel.
  *  @return The main panel.
  */
  public Container getMainPanel()
  {
    return tMainPanel;
  }

  /**
  *  Tests if the state of the dialog is valid.
  *  @return True if dialog state is valid.
  */
  public boolean isValid()
  {

    ActionEvent              tActionEvent;
    JComponent               tComponent;
    TAbstractInputVerifier   tInputVerifier;
    boolean                  gIsValid = true;

    for (int I = 0; ( I < tVerifiedComponentList.size() ) ;++I)
    {
      tComponent     = (JComponent) tVerifiedComponentList.get (I);
      tInputVerifier = (TAbstractInputVerifier) tComponent.getInputVerifier();
      if ( !tInputVerifier.verify (tComponent) )
      {
        gIsValid     = false;
        tActionEvent = new ActionEvent (this, tInputVerifier.hashCode(), tInputVerifier.toString());
        tController.actionPerformed (tActionEvent);
        tComponent.requestFocus();
        JOptionPane.showMessageDialog (this, tInputVerifier.getErrorMessage());
        break;
      }
    }
    return gIsValid;

  }  // isValid()

}  // class TAbstractDialog
