/*
*  Name:      TAbstractSimpleTableModel.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Abstract simple table model
*  Date:      $Date: 2005-01-19 16:57:39 +0100 (mié, 19 ene 2005) $
*  Revision:  $Revision: 10620 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.awt.Dimension;
import java.util.List;
import javax.swing.table.AbstractTableModel;


/**
*  Abstract simple table model.  It also implements the \a ITableAppearanceModel
*  to provide default implementations for that interface.
*/
public abstract class TAbstractSimpleTableModel
  extends    AbstractTableModel
  implements ITableAppearanceModel
{

  ///  List of objects. It must be a 'java.util.List' to allow indexed access.
  protected List   tList;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new object.
  *  @param tLIST List of objects.
  */
  public TAbstractSimpleTableModel (List tLIST)
  {
    tList = tLIST;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns  the most  specific  superclass  for all  the cell  values in  the
  *  column.  This is  used by the \a JTable to  set up a default renderer  and
  *  editor for the column.
  *  @param iCOLUMN The index of the column.
  *  @return The common ancestor class of the object values in the model.
  *  @see javax.swing.table.TableModel#getColumnClass (int)
  */
  public Class getColumnClass (int iCOLUMN)
  {
    Class   tClass = String.class;

    if ( getRowCount() > 0 )
    {
      tClass = getValueAt (0, iCOLUMN).getClass();
    }
    return tClass;
  }

  /**
  *  Returns the minimum width for a table column.
  *  @param iCOLUMN Column index on the table.
  *  @see org.mpcl.nui.table.ITableAppearanceModel#getColumnMinWidth()
  */
  public int getColumnMinWidth (int iCOLUMN)
  {
    //
    //  Value inspired by javax.swing.table.TableColumn#TableColumn() defaults.
    //
    return 75;
  }

  /**
  *  Returns the container.
  *  @return The container.
  */
  public List getList()
  {
    return tList;
  }

  /**
  *  Returns the preferred scrollable viewport size for the table.  It computes
  *  the width as the sum of the column minimum widths plus inner borders.
  *  @return The preferred scrollable viewport size.
  *  @see org.mpcl.nui.table.ITableAppearanceModel#getPreferredScrollableViewportSize()
  */
  public Dimension getPreferredScrollableViewportSize()
  {
    int   iNumberOfColumns;
    int   iPreferredWidth = -1;

    iNumberOfColumns = getColumnCount();
    for (int I = 0; ( I < iNumberOfColumns ) ;++I)
    {
      iPreferredWidth += getColumnMinWidth (I) + 1;
    }
    return new Dimension (Math.max (0, iPreferredWidth), 400);
  }

  /**
  *  Returns the  number of rows in the model.  A \a JTable uses this method to
  *  determine how many rows it should display. This method should be quick, as
  *  it is called frequently during rendering.
  *  @return The number of rows in the model.
  *  @see javax.swing.table.TableModel#getRowCount()
  */
  public int getRowCount()
  {
    return tList.size();
  }

  /**
  *  Gets the tool-tip for the cell at row \a iROW and column \a iCOLUMN.
  *  @param iROW    Row index.
  *  @param iCOLUMN Column index.
  *  @return The tool-tip text.
  *  @see org.mpcl.nui.table.ITableAppearanceModel#getToolTipText()
  */
  public String getToolTipText (int iROW, int iCOLUMN)
  {
    Object   tValue;
    String   yToolTipText = null;

    tValue = getValueAt (iROW, iCOLUMN);
    if ( tValue != null )
    {
      yToolTipText = tValue.toString();
    }
    return yToolTipText;
  }

}  // class TAbstractSimpleTableModel
