/*
*  Name:      TDecoratorTableCellEditor.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Table cell editor that decorates a delegate table cell editor
*  Date:      $Date: 2004-10-31 22:59:36 +0100 (dom, 31 oct 2004) $
*  Revision:  $Revision: 10387 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.awt.Component;
import java.util.EventObject;
import javax.swing.AbstractCellEditor;
import javax.swing.JTable;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;


/**
*  Table cell editor that decorates a delegate table cell editor. It implements
*  the Decorator desing pattern.
*/
public class TDecoratorTableCellEditor implements TableCellEditor
{

  ///  The delegate table cell editor.
  private TableCellEditor   tDelegateTableCellEditor;

  ///  Table cell decorator.
  private ITableCellDecorator   tTableCellDecorator;

  
  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tDELEGATE_TABLE_CELL_EDITOR The delegate table cell editor.
  *  @param tTABLE_CELL_DECORATOR       Table cell decorator.
  */
  public TDecoratorTableCellEditor ( TableCellEditor     tDELEGATE_TABLE_CELL_EDITOR ,
                                     ITableCellDecorator tTABLE_CELL_DECORATOR       )
  {
    tDelegateTableCellEditor = tDELEGATE_TABLE_CELL_EDITOR;
    tTableCellDecorator      = tTABLE_CELL_DECORATOR;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Tells the editor  to cancel  editing and not accept  any partially edited
  *  value.
  *  @see javax.swing.CellEditor#cancelCellEditing()
  */
  public void cancelCellEditing()
  {
    tDelegateTableCellEditor.cancelCellEditing();
  }

  /**
  *  Returns the value contained in the editor. 
  *  @return The value contained in the editor.
  *  @see javax.swing.CellEditor#getEditorValue()
  */
  public Object getCellEditorValue()
  {
    return tDelegateTableCellEditor.getCellEditorValue();
  }

  /**
  *  Sets an initial value for the editor.  This will  cause the editor to stop
  *  editing and lose any  partially edited value if the editor is editing when
  *  this method  is called.  It returns the component  that should be added to
  *  the client's Component hierarchy. Once installed in the client's hierarchy
  *  this component will then be able to draw and receive user input.
  *  @param tTABLE
  *         The \a JTable that is asking the editor to edit; It can be null.
  *  @param tVALUE
  *         The value of the cell to be edited; it is up to the specific editor
  *         to  interpret  and draw  the value.  For example,  if value is  the
  *         string "true",  it could be  rendered as a  string or it  could  be
  *         rendered as a check box that is checked. null is a valid value.
  *  @param gIS_SELECTED
  *         True if the cell is to be rendered with highlighting.
  *  @param iROW
  *         The row of the cell being edited.
  *  @param iCOLUMN
  *         The column of the cell being edited.
  *  @return The component for editing.
  *  @see javax.swing.table.TableCellEditor#getTableCellEditorComponent()
  */
  public Component getTableCellEditorComponent ( JTable   tTABLE       ,
                                                 Object   tVALUE       ,
                                                 boolean  gIS_SELECTED ,
                                                 int      iROW         ,
                                                 int      iCOLUMN      )
  {

    Component   tResultComponent;

    tResultComponent =
      tDelegateTableCellEditor.getTableCellEditorComponent ( tTABLE       ,
                                                             tVALUE       ,
                                                             gIS_SELECTED ,
                                                             iROW         ,
                                                             iCOLUMN      );
    tResultComponent =
      tTableCellDecorator.getTableCellComponent ( tResultComponent ,
                                                  tTABLE           ,
                                                  tVALUE           ,
                                                  gIS_SELECTED     ,
                                                  true             ,
                                                  iROW             ,
                                                  iCOLUMN          );
    return tResultComponent;

  }  // getTableCellEditorComponent()

  /**
  *  Asks the editor if it can start editing using \a tEVENT_OBJECT. This event
  *  is in the invoking component coordinate system.  The editor can not assume
  *  the Component  returned by \a getCellEditorComponent() is installed.  This
  *  method is intended for the use of  client to avoid  the cost of setting up
  *  and installing the editor component if editing is not possible. If editing
  *  can be started this method returns true.
  *  @param tEVENT_OBJECT
  *         The event the editor should use to consider whether to begin
  *         editing or not.
  *  @return True if editing can be started.
  *  @see #shouldSelectCell (java.util.EventObject)
  *  @see javax.swing.CellEditor#isCellEditable()
  */
  public boolean isCellEditable (EventObject tEVENT_OBJECT)
  {
    return tDelegateTableCellEditor.isCellEditable (tEVENT_OBJECT);
  }

  /**
  *  Returns true if the editing cell should be selected, false otherwise.
  *  Typically, the return value is true, because is most cases the editing
  *  cell should be selected. However, it is useful to return false to keep the
  *  selection from changing for some types of edits. eg. A table that contains
  *  a column of check boxes,  the user might want to be able  to change  those
  *  checkboxes without altering the selection.  (See Netscape Communicator for
  *  just such an example) Of course,  it is up to the client  of the editor to
  *  use the return value, but it doesn't need to if it doesn't want to. 
  *  @param tEVENT_OBJECT The event the editor should use to start editing.
  *  @return True if  the editor  would like the  editing cell to  be selected;
  *          otherwise returns false.
  *  @see #isCellEditable (java.util.EventObject)
  *  @see javax.swing.CellEditor#shouldSelectCell()
  */
  public boolean shouldSelectCell (EventObject tEVENT_OBJECT)
  {
    return tDelegateTableCellEditor.shouldSelectCell (tEVENT_OBJECT);
  }

  /**
  *  Tells the editor to stop  editing and accept any partially edited value as
  *  the value of  the editor.  The editor  returns false  if editing  was  not
  *  stopped;  this is useful  for  editors that  validate and  can not  accept
  *  invalid entries.
  *  @return True if editing was stopped; false otherwise.
  *  @see javax.swing.CellEditor#stopCellEditing()
  */
  public boolean stopCellEditing()
  {
    return tDelegateTableCellEditor.stopCellEditing();
  }

  /**
  *  Adds a listener to the list that's notified when the editor stops, or
  *  cancels editing. 
  *  @param tCELL_EDITOR_LISTENER The \a CellEditorListener object.
  *  @see javax.swing.CellEditor#addCellEditorListener()
  */
  public void addCellEditorListener (CellEditorListener tCELL_EDITOR_LISTENER)
  {
    tDelegateTableCellEditor.addCellEditorListener (tCELL_EDITOR_LISTENER);
  }

  /**
  *  Removes a listener from the list that's notified.
  *  @param tCELL_EDITOR_LISTENER The \a CellEditorListener object.
  *  @see javax.swing.CellEditor#removeCellEditorListener()
  */
  public void removeCellEditorListener (CellEditorListener tCELL_EDITOR_LISTENER)
  {
    tDelegateTableCellEditor.removeCellEditorListener (tCELL_EDITOR_LISTENER);
  }
    
}  // class TDecoratorTableCellEditor
