/*
*  Name:      TPercentTableCellEditor.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Table cell editor for percent numbers
*  Date:      $Date: 2004-11-04 23:39:30 +0100 (jue, 04 nov 2004) $
*  Revision:  $Revision: 10397 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.awt.Color;
import java.awt.Component;
import java.text.ParseException;
import javax.swing.DefaultCellEditor;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import org.mpcl.text.INumberTranslator;
import org.mpcl.text.TNumberTranslator;
import org.mpcl.util.TPercent;


/**
*  Table cell editor for percent objects. It uses a number translator object to
*  convert them  to strings.  It  removes the percent  symbol because the units
*  must go in column headers in parentheses (Java Look & Feel Guidelines).
*  @see org.mpcl.text.INumberTranslator
*/
public class TPercentTableCellEditor extends DefaultCellEditor
{

  ///  Number translator.
  public INumberTranslator   tNumberTranslator;

  ///  The percent value.
  private TPercent   tPercent;


  //
  //  C O N S T R U C T O R S
  //

  ///  Builds a new instance.
  public TPercentTableCellEditor()
  {
    super (new JTextField());

    tPercent          = null;
    tNumberTranslator = new TNumberTranslator();
    ((JTextField) getComponent()).setHorizontalAlignment (JTextField.RIGHT);
  }

  /**
  *  Tells the editor to stop editing and  accept any partially edited value as
  *  the  value of the editor.  The  editor  returns false  if editing  was not
  *  stopped;  this is  useful for  editors that  validate and  can not  accept
  *  invalid entries.
  *  @return True if editing was stopped; false otherwise.
  *  @see javax.swing.CellEditor#stopCellEditing()
  */
  public boolean stopCellEditing()
  {

    boolean   gResult;
    String    yValue;

    gResult = false;
    yValue  = (String) super.getCellEditorValue();
    if ( yValue.trim().length() == 0 )
    {
      tPercent = null;
      gResult  = super.stopCellEditing();
    }
    else
    {
      try
      {
        tPercent = new TPercent (tNumberTranslator.toDouble (yValue));
        gResult  = super.stopCellEditing();
      }
      catch (final ParseException ktEXCEPTION)
      {
        ((JComponent) getComponent()).setBorder (new LineBorder (Color.red));
        gResult = false;
      }
    }
    return gResult;

  }  // stopCellEditing()


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the value contained in the editor.
  *  @return The value contained in the editor.
  *  @see javax.swing.CellEditor#getCellEditorValue()
  */
  public Object getCellEditorValue()
  {
    return tPercent;
  }

  /**
  *  Sets an initial value for the  editor.  This will cause the editor to stop
  *  editing (\a stopEditing) and lose any partially edited value if the editor
  *  is editing when this method is called.  Returns the  component that should
  *  be added  to the  client's  Component hierarchy.  Once  installed  in  the
  *  client's hierarchy  this component will then  be able to draw  and receive
  *  user input. 
  *  @param tTABLE
  *         The JTable object that is asking the editor to edit; can be null.
  *  @param tVALUE
  *         The value of the cell to be edited; it is up to the specific editor
  *         to interpret and draw the value. For example, if \a tVALUE is the
  *         string "true", it could be rendered as a string or it could be
  *         rendered as a check-box that is checked. null is a valid value.
  *  @param gIS_SELECTED
  *         True if the cell is to be rendered with highlighting.
  *  @param iROW
  *         The row of the cell being edited.
  *  @param iCOLUMN
  *         The column of the cell being edited.
  *  @return The component for editing.
  *  @see javax.swing.table.TableCellEditor#getTableCellEditorComponent()
  */
  public Component getTableCellEditorComponent ( JTable  tTABLE       ,
                                                 Object  tVALUE       ,
                                                 boolean gIS_SELECTED ,
                                                 int     iROW         ,
                                                 int     iCOLUMN      )
  {
    String   yValue;

    tPercent = null;
    if ( tVALUE == null )
    {
      yValue = null;
    }
    else
    {
      yValue = tNumberTranslator.toString ((TPercent) tVALUE);
    }
    ((JComponent) getComponent()).setBorder (new LineBorder (Color.black));
    return super.getTableCellEditorComponent (tTABLE, tVALUE, gIS_SELECTED, iROW, iCOLUMN);
  }

}  // class TPercentTableCellEditor
