/*
*  Name:        TAbstractTreeTableModel.java
*  Author:      Philip Milne
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     Abstract tree-table model.
*  Date:        $Date: 2004-10-11 00:59:30 +0200 (lun, 11 oct 2004) $
*  Revision:    $Revision: 10229 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import java.awt.Dimension;
import javax.swing.event.EventListenerList;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

 
/**
*  An abstract implementation of the \a ITreeTableModel interface, handling the
*  list of  listeners.   The next  methods are  left to be  implemented in  the
*  subclass:
*    
*    <code>public Object getChild (Object tPARENT_NODE, int iINDEX);</code>
*    <code>public int getChildCount (Object tPARENT_NODE);</code>
*    <code>public int getColumnCount();</code>
*    <code>public String getColumnName (Object tNODE, int iCOLUMN);</code>
*    <code>public Object getValueAt (Object tNODE, int iCOLUMN);</code>
*/
public abstract class TAbstractTreeTableModel
  implements ITreeTableModel, ITreeTableAppearanceModel
{

  ///  The listener list.
  protected EventListenerList   tEventListenerList;

  ///  The root node.
  protected TreeNode   tRootTreeNode;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tROOT_TREE_NODE The tree root node.
  */
  public TAbstractTreeTableModel (TreeNode tROOT_TREE_NODE)
  {
    tEventListenerList = new EventListenerList();
    tRootTreeNode      = tROOT_TREE_NODE;
  }

  /**
  *  Adds a listener for the \a TreeModelEvent posted after the tree changes.
  *  @param tTREE_MODEL_LISTENER The listener to add.
  *  @see javax.swing.event.TreeModelListener#addTreeModelListener
  */
  public void addTreeModelListener (TreeModelListener tTREE_MODEL_LISTENER)
  {
    tEventListenerList.add (TreeModelListener.class, tTREE_MODEL_LISTENER);
  }

  /**
  *  Removes a listener previously added with \a addTreeModelListener().
  *  @param tTREE_MODEL_LISTENER The listener to remove.
  *  @see javax.swing.event.TreeModelListener#removeTreeModelListener
  */
  public void removeTreeModelListener (TreeModelListener tTREE_MODEL_LISTENER)
  {
    tEventListenerList.remove (TreeModelListener.class, tTREE_MODEL_LISTENER);
  }

  /**
  *  Sets the root  to \a tROOT_TREE_NODE.  A null root  implies the tree is to
  *  display nothing, and is legal.
  *  @param tROOT_TREE_NODE Root tree node.
  */
  public void setRoot (TreeNode tROOT_TREE_NODE)
  {
    tRootTreeNode = tROOT_TREE_NODE;
  }

  /**
  *  Sets the value for node \a tNODE, at row \a iROW and column \a iCOLUMN.
  *  @param tVALUE  The new value.
  *  @param tNODE   The tree node.
  *  @param iROW    The index of the row.
  *  @param iCOLUMN The index of the column.
  *  @see ITreeTableModel#setValueAt
  */
  public void setValueAt (Object tVALUE, Object tNODE, int iROW, int iCOLUMN) {}

  /**
  *  Messaged  when the user has  altered the value for the item  identified by
  *  \a tTREE_PATH  to  \a tNEW_VALUE.  If \a tNEW_VALUE  signifies a truly new
  *  value, the model should post a \a treeNodesChanged event. 
  *  @param tTREE_PATH The path to the node that the user has altered.
  *  @param tNEW_VALUE The new value from the \a TreeCellEditor.
  *  @see javax.swing.tree.TreeModel#valueForPathChanged
  */
  public void valueForPathChanged (TreePath tTREE_PATH, Object tNEW_VALUE) {}


  //
  //   S E L E C T O R S
  //

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tTREE_PATH Tree path.
  */
  public void fireTreeNodesChanged (TreePath tTREE_PATH)
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (this, tTREE_PATH);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeNodesChanged (tTreeModelEvent);
      }          
    }

  }  // fireTreeNodesChanged()

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tSOURCE
  *         The object  responsible  for  generating the  event  (typically the
  *         creator of the event object passes this for its value).
  *  @param atPATH
  *         An  array of \a Object  identifying  the path to the  parent of the
  *         modified  item(s),  where the  first  element of  the array  is the
  *         object stored at the root node  and the last  element is the object
  *         stored at the parent node.
  *  @param aiCHILD_INDICES
  *         An array of \a int that  specifies the index  values of the removed
  *         items. The indices must be in sorted order, from lowest to highest.
  *  @param atCHILDREN
  *         An array of \a Object containing the inserted,  removed, or changed
  *         objects.
  */
  public void fireTreeNodesChanged ( Object   tSOURCE         ,
                                     Object[] atPATH          ,
                                     int[]    aiCHILD_INDICES ,
                                     Object[] atCHILDREN      )
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (tSOURCE, atPATH, aiCHILD_INDICES, atCHILDREN);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeNodesChanged (tTreeModelEvent);
      }          
    }

  }  // fireTreeNodesChanged()

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tTREE_PATH Tree path.
  */
  public void fireTreeNodesInserted (TreePath tTREE_PATH)
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (this, tTREE_PATH);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeNodesInserted (tTreeModelEvent);
      }          
    }

  }  // fireTreeNodesInserted()

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tSOURCE
  *         The object  responsible  for  generating the  event  (typically the
  *         creator of the event object passes this for its value).
  *  @param atPATH
  *         An  array of \a Object  identifying  the path to the  parent of the
  *         modified  item(s),  where the  first  element of  the array  is the
  *         object stored at the root node  and the last  element is the object
  *         stored at the parent node.
  *  @param aiCHILD_INDICES
  *         An array of \a int that  specifies the index  values of the removed
  *         items. The indices must be in sorted order, from lowest to highest.
  *  @param atCHILDREN
  *         An array of \a Object containing the inserted,  removed, or changed
  *         objects.
  */
  public void fireTreeNodesInserted ( Object   tSOURCE         ,
                                      Object[] atPATH          ,
                                      int[]    aiCHILD_INDICES ,
                                      Object[] atCHILDREN      )
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (tSOURCE, atPATH, aiCHILD_INDICES, atCHILDREN);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeNodesInserted (tTreeModelEvent);
      }
    }

  }  // fireTreeNodesInserted()

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tTREE_PATH Tree path.
  */
  public void fireTreeNodesRemoved (TreePath tTREE_PATH)
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (this, tTREE_PATH);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeNodesRemoved (tTreeModelEvent);
      }          
    }

  }  // fireTreeNodesRemoved()

  /*
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tSOURCE
  *         The object  responsible  for  generating the  event  (typically the
  *         creator of the event object passes this for its value).
  *  @param atPATH
  *         An  array of \a Object  identifying  the path to the  parent of the
  *         modified  item(s),  where the  first  element of  the array  is the
  *         object stored at the root node  and the last  element is the object
  *         stored at the parent node.
  *  @param aiCHILD_INDICES
  *         An array of \a int that  specifies the index  values of the removed
  *         items. The indices must be in sorted order, from lowest to highest.
  *  @param atCHILDREN
  *         An array of \a Object containing the inserted,  removed, or changed
  *         objects.
  */
  public void fireTreeNodesRemoved ( Object   tSOURCE         ,
                                     Object[] atPATH          ,
                                     int[]    aiCHILD_INDICES ,
                                     Object[] atCHILDREN      )
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (tSOURCE, atPATH, aiCHILD_INDICES, atCHILDREN);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeNodesRemoved (tTreeModelEvent);
      }
    }

  }  // fireTreeNodesRemoved()

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tTREE_PATH Tree path.
  */
  public void fireTreeStructureChanged (TreePath tTREE_PATH)
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (this, tTREE_PATH);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeStructureChanged (tTreeModelEvent);
      }          
    }

  }  // fireTreeStructureChanged()

  /**
  *  Notifies all listeners  that have registered interest for  notification on
  *  this event type.
  *  @param tSOURCE
  *         The object  responsible  for  generating the  event  (typically the
  *         creator of the event object passes this for its value).
  *  @param atPATH
  *         An  array of \a Object  identifying  the path to the  parent of the
  *         modified  item(s),  where the  first  element of  the array  is the
  *         object stored at the root node  and the last  element is the object
  *         stored at the parent node.
  *  @param aiCHILD_INDICES
  *         An array of \a int that  specifies the index  values of the removed
  *         items. The indices must be in sorted order, from lowest to highest.
  *  @param atCHILDREN
  *         An array of \a Object containing the inserted,  removed, or changed
  *         objects.
  */
  public void fireTreeStructureChanged ( Object   tSOURCE         ,
                                         Object[] atPATH          ,
                                         int[]    aiCHILD_INDICES ,
                                         Object[] atCHILDREN      )
  {

    Object[]         aListenerTypeListenerPair;
    TreeModelEvent   tTreeModelEvent = null;

    //
    //  The getListenerList()  method is guaranteed to return a non-null array.
    //  Processes  the  listeners  last  to first,  notifying  those  that  are
    //  interested in this event.  The \a ChangeEvent object is created lazily.
    //
    aListenerTypeListenerPair = tEventListenerList.getListenerList();
    for (int I = aListenerTypeListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( aListenerTypeListenerPair [I] == TreeModelListener.class )
      {
        if ( tTreeModelEvent == null )
        {
          tTreeModelEvent = new TreeModelEvent (tSOURCE, atPATH, aiCHILD_INDICES, atCHILDREN);
        }
        ((TreeModelListener) aListenerTypeListenerPair [I + 1]).treeStructureChanged (tTreeModelEvent);
      }
    }

  }  // fireTreeStructureChanged()

  /**
  *  Returns  the class of  the cell value for a node in the column \a iCOLUMN.
  *  This is used  by the \a JTable to set up a default renderer and editor for
  *  the column.
  *  @param tNODE   The tree node.
  *  @param iCOLUMN The index of the column.
  *  @return The class of the object value in the model.
  *  @see ITreeTableModel#getCellClass()
  */
  public Class getCellClass (Object tNODE, int iCOLUMN)
  {
    return Object.class;
  }

  /**
  *  Returns the  most  specific  superclass for all  the  cell  values in  the
  *  column.  This is  used by the  \a JTable to  set up a default renderer and
  *  editor for the column.
  *  @param iCOLUMN The index of the column.
  *  @return The common ancestor class of the object values in the model.
  *  @see ITreeTableModel#getColumnClass()
  */
  public Class getColumnClass (int iCOLUMN)
  {
    return Object.class;
  }

  /**
  *  Returns the minimum width for a table column.
  *  @param iCOLUMN Column index on the table.
  *  @see org.mpcl.nui.table.ITableAppearanceModel#getColumnMinWidth()
  */
  public int getColumnMinWidth (int iCOLUMN)
  {
    //
    //  Value inspired by javax.swing.table.TableColumn#TableColumn() defaults.
    //
    return 75;
  }

  /**
  *  Returns  the  index of  child in  parent.  If  \a tPARENT_NODE is  null or
  *  \a tCHILD is null, it returns -1.
  *  @param tPARENT_NODE A node in the tree, obtained from this data source.
  *  @param tCHILD_NODE  The node we are interested in.
  *  @return The  index of  the  CHILD_NODE in  the  parent,  or -1  if  either
  *          tCHILD_NODE or tPARENT_NODE are null.
  */
  public int getIndexOfChild (Object tPARENT_NODE, Object tCHILD_NODE)
  {
    int   iChildNodeIndex = -1;

    for (int I = 0; ( I < getChildCount (tPARENT_NODE) ) ;++I)
    {
      if ( getChild (tPARENT_NODE, I).equals (tCHILD_NODE) )
      {
        iChildNodeIndex = I;
        break;
      }
    }
    return iChildNodeIndex;
  }

  /**
  *  Returns the preferred scrollable viewport size for the table.
  *  @return The preferred scrollable viewport size.
  *  @see org.mpcl.nui.table.ITableAppearanceModel#getPreferredScrollableViewportSize()
  */
  public Dimension getPreferredScrollableViewportSize()
  {
    //
    //  Value inspired by javax.swing.JTable#initializeLocalVars() defaults.
    //
    return new Dimension (450, 400);
  }

  /**
  *  Returns the root of the tree. Returns null only if the tree has no nodes.
  *  @return The root of the tree.
  *  @see javax.swing.tree.TreeModel#getRoot
  */
  public Object getRoot()
  {
    return tRootTreeNode;
  }

  /**
  *  Gets the tool-tip for the node \a tNODE, at column number \a iCOLUMN.
  *  @param tNODE   The tree node.
  *  @param iCOLUMN Column index.
  *  @return The tool-tip.
  */
  public String getToolTipText (Object tNODE, int iCOLUMN)
  {
    return null;
  }

  /**
  *  Indicates whether the value for node \a tNODE, at column number \a iCOLUMN
  *  is editable.  By default, it makes the column with the tree in it the only
  *  editable one.  Making this column editable causes the \a JTable to forward
  *  mouse and keyboard  events in the tree column  to the underlying \a JTree.
  *  @param tNODE   The tree node.
  *  @param iCOLUMN The index of the column.
  *  @return True if the node is editable
  *  @see ITreeTableModel#isCellEditable
  */ 
  public boolean isCellEditable (Object tNODE, int iCOLUMN)
  {
    return ( getColumnClass (iCOLUMN) == ITreeTableModel.class );
  }

  /**
  *  Returns true if node is a leaf.  It is possible for this  method to return
  *  false  even if node  has no children.  A directory  in a  filesystem,  for
  *  example, may contain no files;  the node representing the directory is not
  *  a leaf, but it also has no children.
  *  @param tNODE A node in the tree, obtained from this data source.
  *  @return True if node is a leaf.
  *  @see javax.swing.tree.TreeModel#isLeaf
  */
  public boolean isLeaf (Object tNODE)
  {
    return ( getChildCount (tNODE) == 0 );
  }

  /**
  *  Returns true if the last node in the path is editable.
  *  @param tTREE_PATH The tree-path object.
  *  @see javax.swing.JTree#isPathEditable()
  *  @see org.mpcl.nui.treetable.ITreeTableModel#isCellEditable()
  */
  public boolean isPathEditable (TreePath tTREE_PATH)
  {
    return true;
  }

}  // class TAbstractTreeTableModel
