/*
*  Name:      TTreeTableCellRenderer.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Cell renderer for tables and trees.
*  Date:      $Date: 2004-10-01 02:01:50 +0200 (vie, 01 oct 2004) $
*  Revision:  $Revision: 10189 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import java.awt.Component;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import org.mpcl.nui.TTreeTable;


///  Cell renderer for the tree-table tree column.
public class TTreeTableCellRenderer implements ITreeTableCellRenderer
{

  ///  Tree.
  protected JTree   tTree;

  ///  Tree cell renderer.
  protected DefaultTreeCellRenderer   tTreeCellRenderer;

  ///  Tree table.
  protected TTreeTable   tTreeTable;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tTREE_TABLE The \a TTreeTable object.
  */
  public TTreeTableCellRenderer (TTreeTable tTREE_TABLE)
  {
    tTreeTable        = tTREE_TABLE;
    tTree             = tTreeTable.getTree();
    tTreeCellRenderer = new DefaultTreeCellRenderer();
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the  component used for  drawing the cell.  This method is used to
  *  configure the renderer appropriately before drawing.
  *  @param tTABLE
  *         The \a JTable that is asking the renderer to draw; can be null.
  *  @param tVALUE
  *         The value  of the cell to be  rendered.  It is  up to the  specific
  *         renderer to interpret and draw the value. For example, if \a tVALUE
  *         is the string "true",  it could be rendered as a string or it could
  *         be rendered as a check box that is checked. It can be \a null.
  *  @param gIS_SELECTED
  *         True if the cell is to be rendered with the selection  highlighted;
  *         otherwise false.
  *  @param gHAS_FOCUS
  *         if true,  render cell  appropriately.  For example,  put a  special
  *         border on the cell, if the cell can be edited,  render in the color
  *         used to indicate editing.
  *  @param iROW
  *         The row index of the cell being drawn. When drawing the header, the
  *         value of row is -1.
  *  @param iCOLUMN
  *         The column index of the cell being drawn.
  *  @return The \a Component that the renderer uses to draw the value.
  *  @see ITreeTableCellRenderer#getTreeCellRendererComponent()
  */
  public Component getTableCellRendererComponent ( JTable  tTABLE       ,
                                                   Object  tVALUE       ,
                                                   boolean gIS_SELECTED ,
                                                   boolean gHAS_FOCUS   ,
                                                   int     iROW         ,
                                                   int     iCOLUMN      )
  {
    if ( gIS_SELECTED )
    {
      tTree.setBackground (tTABLE.getSelectionBackground());
    }
    else
    {
      tTree.setBackground (tTABLE.getBackground());
    }
    tTreeTable.setLastVisibleRow (iROW);
    return tTree;
  }

  /**
  *  Sets the value of the  current tree cell to \a tVALUE.  If \a gIS_SELECTED
  *  is true, the cell will be drawn as if selected. If \a gIS_EXPANDED is true
  *  the node is currently expanded. If \a gIS_LEAF is true the node represents
  *  a leaf.  If gHAS_FOCUS is true the node  currently has focus.  \a tTREE is
  *  the \a JTree  and  the  receiver  is  being  configured  for.  Returns the
  *  \a Component that the renderer uses to draw the value.
  *  @param tTREE        Tree where the cell belongs to.
  *  @param tVALUE       Value contained in the cell.
  *  @param gIS_SELECTED True if the cell is selected.
  *  @param gIS_EXPANDED True if the node is expanded.
  *  @param gIS_LEAF     True if the node represents a leaf.
  *  @param iROW         Row of the cell.
  *  @param gHAS_FOCUS   True if the cell has the focus.
  *  @return The \a Component that the renderer uses to draw the value.
  *  @see ITreeTableCellRenderer#getTreeCellRendererComponent()
  */
  public Component getTreeCellRendererComponent ( JTree   tTREE        ,
                                                  Object  tVALUE       ,
                                                  boolean gIS_SELECTED ,
                                                  boolean gIS_EXPANDED ,
                                                  boolean gIS_LEAF     ,
                                                  int     iROW         ,
                                                  boolean gHAS_FOCUS   )
  {
    tTreeCellRenderer.getTreeCellRendererComponent ( tTREE        ,
                                                     tVALUE       ,
                                                     gIS_SELECTED ,
                                                     gIS_EXPANDED ,
                                                     gIS_LEAF     ,
                                                     iROW         ,
                                                     gHAS_FOCUS   );
    tTreeCellRenderer.setTextNonSelectionColor       (tTreeTable.getForeground());
    tTreeCellRenderer.setBackgroundNonSelectionColor (tTreeTable.getBackground());
    return tTreeCellRenderer;
  }

}  // class TTreeTableCellRenderer
