/*
*  Name:      TDurationFormat.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Class that adds support for formatting duration
*  Date:      $Date: 2004-11-13 03:36:10 +0100 (sáb, 13 nov 2004) $
*  Revision:  $Revision: 10441 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for text related classes.
package org.mpcl.text;

import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.text.Format;
import java.text.NumberFormat;
import java.text.ParsePosition;


/**
*  Class  to  convert from  time duration to  their  string  representation and
*  vice-versa.
*/
public class TDurationFormat extends Format
{

  ///  Number format used for each group of time values.
  public NumberFormat   tNumberFormat;


  //
  //  C O N S T R U C T O R S
  //

  ///  Builds a new instance. Private because this is really a factory.
  private TDurationFormat()
  {
    tNumberFormat = new DecimalFormat ("00");
  }


  //
  //  C O N S T R U C T O R S
  //

  public static TDurationFormat _getDurationInstance()
  {
    return new TDurationFormat();
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Formats a number representing  a duration  in milliseconds and appends the
  *  resulting text to the given string buffer.
  *  @param tOBJECT
  *         The number of milliseconds to format.
  *  @param yAPPEND_TO_TARGET
  *         The \a StringBuffer to which the formatted text is to be appended.
  *  @param tFIELD_POSITION
  *         The offsets of the alignment field.
  *  @pre ( tOBJECT instanceof Number )
  *  @return The value passed in as \a yAPPEND_TO_TARGET.
  *  @exception IllegalArgumentException If tOBJECT is null or no an instance Number.
  *  @see java.text.FieldPosition
  */
  public StringBuffer format ( Object        tOBJECT           ,
                               StringBuffer  yAPPEND_TO_TARGET ,
                               FieldPosition tFIELD_POSITION   )
  {

    int    iHours;
    int    iMinutes;
    int    iSeconds;
    long   lTotalSeconds;

    if ( tOBJECT instanceof Number )
    {
      lTotalSeconds = ((Number) tOBJECT).longValue() / 1000;
      iSeconds      = (int) (lTotalSeconds % 60);
      iMinutes      = (int) ((lTotalSeconds / 60) % 60);
      iHours        = (int) (lTotalSeconds / 3600);
      yAPPEND_TO_TARGET.append (iHours);
      yAPPEND_TO_TARGET.append (":");
      tNumberFormat.format (new Integer (iMinutes), yAPPEND_TO_TARGET, tFIELD_POSITION);
      yAPPEND_TO_TARGET.append (":");
      tNumberFormat.format (new Integer (iSeconds), yAPPEND_TO_TARGET, tFIELD_POSITION);
    }
    else
    {
      throw new IllegalArgumentException ("cannot format given Object as a Number");
    }
    return yAPPEND_TO_TARGET;

  }  // format()

  /**
  *  Parses text from a  string to  produce an object.  The method  attempts to
  *  parse text starting at the index  given by \a tPARSE_POSITION.  If parsing
  *  succeeds,  then the index  of \a tPARSE_POSITION is  updated to the  index
  *  after  the last  character used  (parsing does  not  necessarily  use  all
  *  characters up  to  the  end of  the  string),  and  the  parsed  object is
  *  returned.  The  updated  \a tPARSE_POSITION  can be  used to  indicate the
  *  starting point for the next call to this method.  If an error occurs, then
  *  the  index  of \a tPARSE_POSITION is  not  changed,  the  error  index  of
  *  \a tPARSE_POSITION  is set to  the index of  the character where the error
  *  occurred, and null is returned.
  *  @param ySOURCE
  *         A \a String, part of which should be parsed.
  *  @param tPARSE_POSITION
  *         A \a ParsePosition object with index and error index information as
  *         described above.
  *  @return The object created.
  */
  public Object parseObject (String ySOURCE, ParsePosition tPARSE_POSITION)
  {
    throw new IllegalArgumentException ("unimplemented method");
  }

}  // class TDurationFormat
